<?php
// tòmò web analytics
// Copyright since 2011 by Yves Goergen
// Website: http://unclassified.de/go/tomo
// Licence: GNU General Public License (GPL) Version 3
// Encoding: UTF-8 (without BOM) (auto-detect: °°°°°)
//
// This file contains the viewer interface.

class TomoViewer
{
	public static $queriesPath = 'queries/';
	
	private static $t0;
	
	public static function Start()
	{
		// Localisation support
		#UnbLocale::Prefer('en-GB');
		#UnbLocale::Prefer('de-DE');
		UnbLocale::AutoPrefer();
		#UnbLocale::SetTimeZone('UTC');

		// Make sure output buffering is active.
		UnbEnvironment::SetupEnvironment();

		// Re-enable error display in testing environment.
		ini_restore('display_errors');

		self::$t0 = microtime(true);

		header('Content-type: text/html; charset=UTF-8');

		echo '<!doctype html>' . "\n";
		echo '<html>';
		echo '<head>';
		echo '<title>' . unbtr('tomo viewer title') . '</title>';
		echo '<link rel="stylesheet" type="text/css" href="css.php">';
		echo '<meta name="robots" content="noindex, nofollow">';
		echo '<link rel="shortcut icon" href="' .
			htmlspecialchars(UnbConfiguration::Get('viewer.shortcut-icon', 'img/favicon.ico')) . '">';
##skip
		echo '<script type="text/javascript" src="coati.js"></script>';
##endskip
		echo '<script type="text/javascript" src="viewer.js"></script>';
		echo '</head>';
		echo '<body>';

		UnbDebug::StartErrorHandling(true);

		// Check viewing access
		$accessCookieName = UnbConfiguration::Get('cookie.name.access', 'tomo_access');
		$accessToken = $_COOKIE[$accessCookieName];
		$password = UnbConfiguration::Get('access.password');
		// The access cookie should only be available inside the tòmò directory, not on the entire website
		// where it might be exposed to cookie-theft attacks.
		$cookiePath = preg_replace('_[^/]*$_', '', $_SERVER['REQUEST_URI']);
		$saltLength = 8;
		
		if ($_POST['logout'])
		{
			$accessToken = null;
			setcookie(
				$accessCookieName,
				'*',
				1,
				$cookiePath);
		}
		
		if (isset($accessToken))
		{
			// Validate given access token
			$salt = substr($accessToken, 0, $saltLength);
			if ($accessToken != $salt . md5($salt . $password))
			{
				// Invalid token, don't use it
				$accessToken = null;
			}
		}
		
		if (!isset($accessToken) && isset($_POST['login_password']))
		{
			if ($_POST['login_password'] == $password)
			{
				// Good, set access cookie.
				// The salt is the first 8 characters of some random md5 hash. This implementation is
				// simple and should be effective enough.
				$salt = substr(md5(mt_rand()), 0, $saltLength);
				$accessToken = $salt . md5($salt . $password);
				
				$expire = $_POST['login_temp'] ?
					0 :
					time() + intval(intval(UnbConfiguration::Get('cookie.months', 12)) * 30.44 * 86400);
				
				setcookie(
					$accessCookieName,
					$accessToken,
					$expire,
					$cookiePath);
			}
		}

		if (!isset($accessToken))
		{
			// Not logged in, show login form
			
			echo '<div id="page-content">';

			echo '<div class="login-intro">';

			echo '<img src="img/globe_chart.png" alt=""><br>';
			echo 'tòmò web analytics';
			
			echo '</div>'; // #login-intro

			echo '<div class="login-form">';
			echo '<form method="post">';
			
			echo unbtr('enter access password') . unbcol() . '<br>';
			
			echo '<input type="password" name="login_password" size="20" autofocus><br>';
			
			echo '<input type="submit" class="call-to-action" value="' . unbtrc('login') . '">';

			echo '<div class="temp"><label title="' . unbtr('login.temporary~') . '"><input type="checkbox" name="login_temp" value="1">' . unbtr('login.temporary') . '</label></div>';

			echo '</form>';
			echo '</div>'; // #login-form
		}
		else
		{
			// Logged in

			echo '<div id="page-header">';

			echo '<div style="float: right; margin-left: 8px;">';
			echo '<form method="post" style="display: inline;">';
			echo '<input type="submit" name="logout" value="' . unbtrc('logout') . '">';
			echo '</form>';
			echo '</div>';
		
			// Now that the HTML page is ready and error handling is set up, initialise the rest.
			UnbDatabase::GetInstance()->Open();
			$db = UnbDatabase::GetInstance();
		
			$tomo = new Tomo();
			
			$lastPurgeTime = Tomo::GetSetting('lastpurgetime', 0);
			if (time() > $lastPurgeTime + 4 * 86400)
			{
				// Time to purge old and unused data lying around in the database.
				set_time_limit(600);
				$tomo->PurgeTempSessionData();
				try
				{
					$tomo->PurgeText();
				}
				catch (UnbDatabaseException $ex)
				{
					echo '<p class="error">Purging texts: ' . $ex->getMessage() . '</p>';
				}
				Tomo::SetSetting('lastpurgetime', time());
				set_time_limit(60);
			}
		
			$lastQuickPurgeTime = Tomo::GetSetting('lastquickpurgetime', 0);
			if (time() > $lastQuickPurgeTime + 60)
			{
				// Time to do a quicker purge run in the database.
				$tomo->PurgeTempPageviews();
				$tomo->UpdateAllExitPages();
				Tomo::SetSetting('lastquickpurgetime', time());
			}
		
			$viewType = $_GET['view'];
		
			// —————————————————————————————————————— VISITOR LOG VIEW ——————————————————————————————————————
		
			if ($viewType == 'visitorlog')
			{
				echo '<a href="?view=index" title="' . unbtrc('overview link') . '" class="roundnavlink">' . GetIcon('roundback') . '</a>';
		
				echo '<h1>' . unbtr('visitor log') . ' <span class="domain">tòmò@' . $_SERVER['SERVER_NAME'] . '</span></h1>';

				$columns = $tomo->GetAllColumnGroups();
		
				// Fill translation dictionary for JavaScript code
				echo '<script type="text/javascript">' . "\n";
				foreach ($columns as $col => $group)
				{
					echo "trdict['column:" . $col . "'] = '" . addslashes(unbtr('column:' . $col, true)) . "';\n";
					echo "trdict['coltype:" . $col . "'] = '" . $tomo->GetColumnType($col) . "';\n";
				}
				echo "trdict['op:between'] = '" . addslashes(unbtr('op:between')) . "';\n";
				echo "trdict['op:isnull'] = '" . addslashes(unbtr('op:isnull')) . "';\n";
				echo "trdict['set'] = '" . addslashes(unbtr('set')) . "';\n";
				echo "trdict['op:contains'] = '" . addslashes(unbtr('op:contains')) . "';\n";
				echo "trdict['op:startswith'] = '" . addslashes(unbtr('op:startswith')) . "';\n";
				echo "trdict['op:endswith'] = '" . addslashes(unbtr('op:endswith')) . "';\n";
				echo "trdict['op:between-and'] = '" . addslashes(unbtr('op:between-and')) . "';\n";
				echo "trdict['remove column'] = '" . addslashes(unbtrc('remove column')) . "';\n";
				echo "trdict['not'] = '" . addslashes(unbtrc('not')) . "';\n";
				echo "trdict['disabled'] = '" . addslashes(unbtrc('disabled')) . "';\n";
				echo "trdict['icon.delete'] = '" . addslashes(GetIcon('delete')) . "';\n";
				echo '</script>';
		
				echo '<div class="tab-header white" id="addColumnsTab" style="margin-top: 10px;">';
				echo '<a href="javascript:undefined" data-tabpage="filterTab">' . unbtrc('filter') . '<span class="shadow"></span></a>';
				echo '<a href="javascript:undefined" data-tabpage="filterOptionsTab">' . unbtrc('options') . '<span class="shadow"></span></a>';
				echo '<span class="grey" style="margin: 0 6px 0 18px;">' . unbtrc('add field') . unbcol() . '</span>';
				$tabLinks = array();
				foreach ($columns as $group)
				{
					if ($group)
					{
						list($group, $subgroup) = explode('/', $group);
						
						switch ($group)
						{
							case 'visit': $sortIndex = 1; break;
							case 'page': $sortIndex = 2; break;
							case 'client': $sortIndex = 3; break;
							case 'vars': $sortIndex = 4; break;
							case 'advanced': $sortIndex = 5; break;
							default: $sortIndex = $group; break;
						}
						if (!isset($tabLinks[$sortIndex]))
						{
							$tabLinks[$sortIndex] = '<a href="javascript:undefined" data-tabpage="' . $group . 'Tab">' . unbtr('colgroup:' . $group, true) . '<span class="shadow"></span></a>';
						}
					}
				}
				ksort($tabLinks);
				foreach ($tabLinks as $tabLink)
				{
					echo $tabLink;
				}
				echo '</div>';
				
				$seenGroups = array();
				foreach ($columns as $group)
				{
					if ($group)
					{
						list($group, $subgroup) = explode('/', $group);
						
						if (!in_array($group, $seenGroups))
						{
							$seenGroups[] = $group;
						
							echo '<div class="tab-page white" id="' . $group . 'Tab">';
							$subgroups = array();
							foreach ($columns as $col => $_colgroup)
							{
								list($colgroup, $colsubgroup) = explode('/', $_colgroup);
						
								if ($colgroup == $group)
								{
									switch ($_colgroup)
									{
										case 'visit/session': $sortIndex = 1; break;
										case 'visit/source': $sortIndex = 2; break;
										case 'visit/search': $sortIndex = 3; break;
										case 'visit/location': $sortIndex = 4; break;
										case 'page/pageview': $sortIndex = 1; break;
										case 'page/address': $sortIndex = 2; break;
										case 'page/action': $sortIndex = 3; break;
										case 'client/browser': $sortIndex = 1; break;
										case 'client/config': $sortIndex = 2; break;
										case 'client/plugins': $sortIndex = 3; break;
										case 'client/screen': $sortIndex = 4; break;
										case 'vars/visitor': $sortIndex = 1; break;
										case 'vars/session': $sortIndex = 2; break;
										case 'vars/pageview': $sortIndex = 3; break;
										case 'advanced/ids': $sortIndex = 1; break;
										case 'advanced/data': $sortIndex = 2; break;
										default: $sortIndex = $_colgroup; break;
									}

									if (!$subgroups[$sortIndex])
									{
										$subgroups[$sortIndex] .= '<span style="display: inline-block; width: 90px;">' . unbtr('colgroup:' . $_colgroup, true) . unbcol() . '</span>';
									}
									
									$subgroups[$sortIndex] .= '<input type="button" class="light column-name" onclick="addFilterColumn(\'' . $col . '\', this.title)" title="' . unbtr('coldesc:' . $col) . '" value="' . unbtr('column:' . $col, true) . '">';
								}
							}
							ksort($subgroups);
							foreach ($subgroups as $subgroup)
							{
								echo '<div style="margin-bottom: 2px;">' . $subgroup . '</div>';
							}
							echo '</div>';
						}
					}
				}
				
				echo '<form method="get">';
				echo '<input type="hidden" name="view" value="visitorlog">';

				function ShowFilterRow($col, $type)
				{
					echo '<tr class="va" id="filter_row_' . $col . '">';

					echo '<td>';
					echo '<a href="javascript:undefined" onclick="removeFilterColumn(\'' . $col . '\')" title="' . unbtrc('remove column') . '">' . GetIcon('delete') . '</a>';
		
					echo '<span title="' . unbtr('coldesc:' . $col) . '">' . unbtr('column:' . $col, true) . '<input type="hidden" name="filtercol[]" value="' . $col . '"></span>';
					echo '</td>';
		
					$not = $_REQUEST[$col . '_not'];
					echo '<td><label><input type="checkbox" name="' . $col . '_not" value="1"' . ($not ? ' checked="checked"' : '') . '>' . unbtrc('not') . '</label></td>';
		
					$op = $_REQUEST[$col . '_op'];
					echo '<td><select name="' . $col . '_op" id="filterop_' . $col . '" onchange="filterOpChanged(this)" data-column="' . $col . '">';
					switch ($type)
					{
						case 'int':
							echo '	<option value="eq"' . ($op == 'eq' ? ' selected="selected"' : '') . '>=</option>';
							echo '	<option value="lt"' . ($op == 'lt' ? ' selected="selected"' : '') . '>&lt;</option>';
							echo '	<option value="lte"' . ($op == 'lte' ? ' selected="selected"' : '') . '>&#x2264;</option>';
							echo '	<option value="gt"' . ($op == 'gt' ? ' selected="selected"' : '') . '>&gt;</option>';
							echo '	<option value="gte"' . ($op == 'gte' ? ' selected="selected"' : '') . '>&#x2265;</option>';
							echo '	<option value="ne"' . ($op == 'ne' ? ' selected="selected"' : '') . '>&#x2260;</option>';
							echo '	<option value="between"' . ($op == 'between' ? ' selected="selected"' : '') . '>' . unbtr('op:between') . '</option>';
							echo '	<option value="isnull"' . ($op == 'isnull' ? ' selected="selected"' : '') . '>' . unbtr('op:isnull') . '</option>';
							break;
						case 'datetime':
							echo '	<option value="eq"' . ($op == 'eq' ? ' selected="selected"' : '') . '>=</option>';
							echo '	<option value="lt"' . ($op == 'lt' ? ' selected="selected"' : '') . '>&lt;</option>';
							echo '	<option value="lte"' . ($op == 'lte' ? ' selected="selected"' : '') . '>&#x2264;</option>';
							echo '	<option value="gt"' . ($op == 'gt' ? ' selected="selected"' : '') . '>&gt;</option>';
							echo '	<option value="gte"' . ($op == 'gte' ? ' selected="selected"' : '') . '>&#x2265;</option>';
							echo '	<option value="ne"' . ($op == 'ne' ? ' selected="selected"' : '') . '>&#x2260;</option>';
							echo '	<option value="between"' . ($op == 'between' ? ' selected="selected"' : '') . '>' . unbtr('op:between') . '</option>';
							echo '	<option value="inday"' . ($op == 'inday' ? ' selected="selected"' : '') . '>' . unbtr('op:inday') . '</option>';
							echo '	<option value="inmonth"' . ($op == 'inmonth' ? ' selected="selected"' : '') . '>' . unbtr('op:inmonth') . '</option>';
							echo '	<option value="inyear"' . ($op == 'inyear' ? ' selected="selected"' : '') . '>' . unbtr('op:inyear') . '</option>';
							echo '	<option value="isnull"' . ($op == 'isnull' ? ' selected="selected"' : '') . '>' . unbtr('op:isnull') . '</option>';
							break;
						case 'bool':
							echo '	<option value="eq"' . ($op == 'eq' ? ' selected="selected"' : '') . '>' . unbtr('set') . '</option>';
							echo '	<option value="isnull"' . ($op == 'isnull' ? ' selected="selected"' : '') . '>' . unbtr('op:isnull') . '</option>';
							break;
						case 'string':
							echo '	<option value="eq"' . ($op == 'eq' ? ' selected="selected"' : '') . '>=</option>';
							echo '	<option value="lt"' . ($op == 'lt' ? ' selected="selected"' : '') . '>&lt;</option>';
							echo '	<option value="lte"' . ($op == 'lte' ? ' selected="selected"' : '') . '>&#x2264;</option>';
							echo '	<option value="gt"' . ($op == 'gt' ? ' selected="selected"' : '') . '>&gt;</option>';
							echo '	<option value="gte"' . ($op == 'gte' ? ' selected="selected"' : '') . '>&#x2265;</option>';
							echo '	<option value="ne"' . ($op == 'ne' ? ' selected="selected"' : '') . '>&#x2260;</option>';
							echo '	<option value="between"' . ($op == 'between' ? ' selected="selected"' : '') . '>' . unbtr('op:between') . '</option>';
							echo '	<option value="contains"' . ($op == 'contains' ? ' selected="selected"' : '') . '>' . unbtr('op:contains') . '</option>';
							echo '	<option value="startswith"' . ($op == 'startswith' ? ' selected="selected"' : '') . '>' . unbtr('op:startswith') . '</option>';
							echo '	<option value="endswith"' . ($op == 'endswith' ? ' selected="selected"' : '') . '>' . unbtr('op:endswith') . '</option>';
							echo '	<option value="isnull"' . ($op == 'isnull' ? ' selected="selected"' : '') . '>' . unbtr('op:isnull') . '</option>';
							break;
					}
					echo '</select></td>';
		
					$value = $_REQUEST[$col . '_value'];
					$value2 = $_REQUEST[$col . '_value2'];
					echo '<td>';
					switch ($type)
					{
						case 'int':
							echo '<span id="input_' . $col . '_value">';
							echo '<input type="text" name="' . $col . '_value" value="' . htmlspecialchars($value) . '" size="20"> ';
							echo '<span id="input_' . $col . '_value2">';
							echo unbtr('op:between-and') . ' <input type="text" name="' . $col . '_value2" value="' . htmlspecialchars($value2) . '" size="20">';
							echo '</span>';
							echo '</span>';
							break;
						case 'datetime':
							echo '<span id="input_' . $col . '_value">';
							echo '<input type="text" name="' . $col . '_value" value="' . htmlspecialchars($value) . '" size="20"> ';
							echo '<span id="input_' . $col . '_value2">';
							echo unbtr('op:between-and') . ' <input type="text" name="' . $col . '_value2" value="' . htmlspecialchars($value2) . '" size="20">';
							echo '</span>';
							echo '</span>';
							break;
						case 'bool':
							echo '<input type="hidden" name="' . $col . '_value" value="1">';
							break;
						case 'string':
							echo '<span id="input_' . $col . '_value">';
							echo '<input type="text" name="' . $col . '_value" value="' . htmlspecialchars($value) . '" size="35"> ';
							echo '<span id="input_' . $col . '_value2">';
							echo unbtr('op:between-and') . ' <input type="text" name="' . $col . '_value2" value="' . htmlspecialchars($value2) . '" size="15">';
							echo '</span>';
							echo '</span>';
							break;
					}
					echo '<script type="text/javascript">' . "\n";
					echo 'filterOpChanged($("#filterop_' . $col . '"));' . "\n";
					echo '</script>';
					echo '</td>';
		
					$disabled = $_REQUEST[$col . '_disabled'];
					echo '<td><label><input type="checkbox" name="' . $col . '_disabled" value="1"' . ($disabled ? ' checked="checked"' : '') . '>' . unbtrc('disabled') . '</label></td>';
		
					echo '</tr>';
				} // function ShowFilterRow
		
				$filterCols = $_REQUEST['filtercol'];
				if (!is_array($filterCols)) $filterCols = array();
		
				if (!$filterCols)
				{
					// No filter is set at all, this could take a long time.
					// Redirect to filtering only the last x days.
					$hours = UnbConfiguration::Get('viewer.visitorlog.default-hours', 2 * 24);
					header('Location: ?' . $_SERVER['QUERY_STRING'] . '&filtercol[]=SessionStartTime&SessionStartTime_op=gt&SessionStartTime_value=' .
						urlencode(gmdate('Y-m-d H:i:s', time() - $hours * 3600)));
					exit();
				}
		
				echo '<div class="tab-page white" id="filterTab">';
				if ($filterCols)
				{
					echo '<div id="filter-table-box">';
					echo '<table id="filter-table">';
					foreach ($filterCols as $col)
					{
						$type = $tomo->GetColumnType($col);
						ShowFilterRow($col, $type);
					}
					echo '</table>';
					echo '</div>'; // #filter-table-box
				}
				else
				{
					echo unbtrc('no filter set');
				}
		
				echo '<input type="submit" class="call-to-action" value="' . unbtrc('update') . '">';
				echo '</div>'; // #filterTab

				echo '<div class="tab-page white" id="filterOptionsTab">';
				
				$limit = intval($_REQUEST['limit']);
				if (!$limit)
					$limit = 1000;
				echo unbtrc('limit visitor log.1') . ' <input type="text" name="limit" value="' . $limit . '"> ' . unbtr('limit visitor log.2') . '<br>';
				
				echo '<label><input type="checkbox" name="reverse" value="1"' . ($_REQUEST['reverse'] ? ' checked' : '') . '>' . unbtrc('reverse visitor log order') . '</label><br>';
				
				echo '<input type="submit" class="call-to-action" value="' . unbtrc('update') . '">';
				echo '</div>'; // #filterOptionsTab
		
				echo '</form>';

				echo '<script type="text/javascript">' . "\n";
				echo 'initializeTabs("addColumnsTab");' . "\n";
				echo '</script>';
				
				echo '</div>'; // #page-header

				echo '<div id="page-content">';

				echo '<table class="with-header" style="width: 100%;">';
		
				echo '<tr>';
				echo '<th>' . unbtrc('time') . '</th>';
				echo '<th>' . unbtrc('location/duration') . '</th>';
				echo '<th>' . unbtrc('client') . '</th>';
				echo '<th>' . unbtrc('source/action') . '</th>';
				echo '</tr>';
		
				$limit = intval($_REQUEST['limit']);
				if (!$limit)
					$limit = 1000;
				
				$pageviews = $tomo->GetPageviews(array_keys($columns), $_REQUEST, $_REQUEST['reverse'], $limit + 1);
				
				// Total values displayed at the end
				$totalVisitors = array();
				$totalSessionCount = 0;
				$totalPageviewCount = 0;
				$totalFirstTime = null;
				$totalLastTime = null;
		
				foreach ($pageviews as $i => $pageview)
				{
					if ($i >= $limit) break;
					
					if ($i == 0 || $pageview['SessionId'] != $pageviews[$i - 1]['SessionId'])
					{
						// New session
						$totalSessionCount++;
						$totalVisitors[$pageview['VisitorId']] = 1;
						$seenUrls = array();
						$prevPageviewTime = $pageview['SessionStartTime'];
						$prevPageviewUpdateTime = 0;
						
						// Statistics for this session
						$pages = array();
						$sessionHitCount = 0;
						$sessionTime = null;
						$prevPVId = 0;
						for ($j = $i; $j < count($pageviews) && $pageviews[$j]['SessionId'] == $pageview['SessionId']; $j++)
						{
							$p = $pageviews[$j];
							if ($p['PageviewId'] != $prevPVId)
							{
								if (!$p['CampaignRedirect'] && !$p['Prerender'])
								{
									$sessionHitCount++;
									if (isset($p['ActivityTimespan']))
									{
										if (!isset($sessionTime))
											$sessionTime = 0;
										$sessionTime += intval($p['ActivityTimespan']);
									}
									if (!in_array($p['UrlPath'], $pages))
										$pages[] = $p['UrlPath'];
								}
								$prevPVId = $p['PageviewId'];
							}
						}
						$sessionPageCount = count($pages);
		
						if ($i > 0)
						{
							echo '<tr class="session-separator-row">';
							echo '<td colspan="4">';
							echo '</td>';
							echo '</tr>';
						}
						
						$cssClass = 'new-session ';
						if (isset($sessionTime) && $sessionTime < 8) $cssClass .= 'short-session ';
						if ($cssClass) $cssClass = ' class="' . trim($cssClass) . '"';
		
						echo '<tr' . $cssClass . '>';
		
						echo '<td class="nbr" title="SessionId: ' . $pageview['SessionId'] . '"><strong>' . unbtm(UnbDatabase::ConvertDateTime($pageview['PageviewTime']), UnbTimeFormat::DateTimeSec) . '</strong><br>';
						if (isset($pageview['UtcOffset']))
						{
							$utcOffset = abs(intval($pageview['UtcOffset']));
							if ($utcOffset % 60 == 0)
							{
								$utcOffset = intval($utcOffset / 60);
							}
							else if ($utcOffset % 60 == 30)
							{
								$utcOffset = intval($utcOffset / 60) . '½';
							}
							else
							{
								$utcOffset = intval($utcOffset / 60) . ':' . str_pad(intval($utcOffset % 60), 2, '0', STR_PAD_LEFT);
							}
							
							echo unbtrc('local') . unbcol() . ' ' .
								unbtm(UnbDatabase::ConvertDateTime($pageview['PageviewTime']) + intval($pageview['UtcOffset']) * 60, UnbTimeFormat::Time, true) .
								' (UTC' . (intval($pageview['UtcOffset']) >= 0 ? '+' : '&minus;') . $utcOffset . ')<br>';
						}
						echo '</td>';
		
						echo '<td class="nbr">';
						$separator = 'separator';
						if (strlen($pageview['CountryCode']))
						{
							$country = unbtr('country:' . $pageview['CountryCode']);
							if (!isset($country))
							{
								$country = GeoIP::GetCountryName($pageview['CountryCode']);
							}
							$region = null;
							if (strlen($pageview['Region']))
							{
								$region = unbtr('region:' . $pageview['CountryCode'] . ':' . $pageview['Region']);
								if (!isset($region))
								{
									$region = $pageview['Region'];
								}
							}
							$city = null;
							if (strlen($pageview['City']))
							{
								$city = unbtr('city:' . $pageview['CountryCode'] . ':' . $pageview['City']);
								if (!isset($city))
								{
									$city = $pageview['City'];
								}
							}
		
							$locStr = '';
							$locTitle = '';
							if (strlen($region))
							{
								$locStr .= ', ' . $region;
								$locTitle .= ", \n" . $region;
							}
							if (strlen($city))
							{
								$locStr .= ', ' . $city;
								$locTitle .= ", \n" . $city;
							}
							$locTitle = $country . ' [' . $pageview['CountryCode'] . ']' . $locTitle;
							$locStr = $pageview['CountryCode'] . $locStr;
		
							echo '<span title="' . $locTitle . '">';
							ShowIcon('country/' . strtolower($pageview['CountryCode']));
							if (isset($pageview['Lat']) && isset($pageview['Lng']))
							{
								echo '<a href="http://maps.google.com/?q=' . urlencode($city) . '+@' . ($pageview['Lat'] / 10000) . ',' . ($pageview['Lng'] / 10000) . '&amp;z=6&amp;t=t">';
							}
							echo str_limit($locStr, 22);
							if (isset($pageview['Lat']) && isset($pageview['Lng']))
							{
								echo '</a>';
							}
							echo '</span>';
						}
						else
						{
							ShowIcon('unknown');
							$separator = 'icon-separator';
						}
						if (strlen($pageview['Language']))
						{
							echo '<span class="' . $separator . '"></span>';
							echo '<span title="' . htmlspecialchars($pageview['Language']) . '">' . htmlspecialchars(str_limit($pageview['Language'], 10)) . '</span>';
						}
						if ($pageview['IPVersion'] == 6 || !$pageview['NewVisitor'])
						{
							echo '<span class="separator"></span>';
							if ($pageview['IPVersion'] == 6)
							{
								ShowIcon('features/ipv6', 'IPv6');
							}
							if (!$pageview['NewVisitor'])
							{
								echo '<a href="?view=visitorlog&amp;filtercol[]=VisitorId&amp;VisitorId_value=' . $pageview['VisitorId'] . '">';
								ShowIcon('returning', unbtrc('returning visitor'));
								echo '</a>';
							}
						}
						echo '<br>' .
							'<span title="' . $pageview['IPAddress'] . '"><a href="?view=ipdiag&amp;ip=' . $pageview['IPAddress'] . '" class="nocolor">' . 
							($pageview['Provider'] ? $pageview['Provider'] : '<em>' . unbtrc('no rdns') . '</em>') . '</a></span>' .
							'</td>';
		
						$browserIcon = '';
						if (preg_match('_^galeon_', $pageview['Browser'])) $browserIcon = 'galeon';
						else if (preg_match('_^gecko/(ff|fennec)_', $pageview['Browser'])) $browserIcon = 'firefox';
						else if (preg_match('_^gecko/seamonkey_', $pageview['Browser'])) $browserIcon = 'seamonkey';
						else if (preg_match('_^gecko/nn_', $pageview['Browser'])) $browserIcon = 'netscape';
						else if (preg_match('_^gecko_', $pageview['Browser'])) $browserIcon = 'gecko';
						else if (preg_match('_^ie/ie/10_', $pageview['Browser'])) $browserIcon = 'ie10';
						else if (preg_match('_^ie/ie/11_', $pageview['Browser'])) $browserIcon = 'ie11';
						else if (preg_match('_^ie/ie/9_', $pageview['Browser'])) $browserIcon = 'ie9';
						else if (preg_match('_^ie/ie/8_', $pageview['Browser'])) $browserIcon = 'ie8';
						else if (preg_match('_^ie/ie/7_', $pageview['Browser'])) $browserIcon = 'ie7';
						else if (preg_match('_^ie/ie/6_', $pageview['Browser'])) $browserIcon = 'ie6';
						else if (preg_match('_^ie/_', $pageview['Browser'])) $browserIcon = 'ie';
						else if (preg_match('_^opera/[1-9][^0-9]_', $pageview['Browser'])) $browserIcon = 'opera9';
						else if (preg_match('_^opera/_', $pageview['Browser'])) $browserIcon = 'opera';
						else if (preg_match('_^webkit/chrome/frame_', $pageview['Browser'])) $browserIcon = 'chromeframe';
						else if (preg_match('_^webkit/chrome/chromium_', $pageview['Browser'])) $browserIcon = 'chromium';
						else if (preg_match('_^webkit/chrome_', $pageview['Browser'])) $browserIcon = 'chrome';
						else if (preg_match('_^webkit/konq_', $pageview['Browser'])) $browserIcon = 'konqueror';
						else if (preg_match('_^webkit/omniweb_', $pageview['Browser'])) $browserIcon = 'omniweb';
						else if (preg_match('_^webkit/safari_', $pageview['Browser'])) $browserIcon = 'safari';
						else if (preg_match('_^webkit_', $pageview['Browser'])) $browserIcon = 'webkit';
						else if (preg_match('_^kindle_', $pageview['Browser'])) $browserIcon = 'kindle';
						else if (preg_match('_^text_', $pageview['Browser'])) $browserIcon = 'text';
		
						$osIcon = '';
						if (preg_match('_^unix/linux/android/4_', $pageview['OperatingSystem'])) $osIcon = 'android4';
						else if (preg_match('_^unix/linux/android/3_', $pageview['OperatingSystem'])) $osIcon = 'android3';
						else if (preg_match('_^unix/linux/android/2_', $pageview['OperatingSystem'])) $osIcon = 'android2';
						else if (preg_match('_^unix/linux/android/1_', $pageview['OperatingSystem'])) $osIcon = 'android1';
						else if (preg_match('_^unix/linux/android_', $pageview['OperatingSystem'])) $osIcon = 'android';
						else if (preg_match('_^blackberry_', $pageview['OperatingSystem'])) $osIcon = 'blackberry';
						else if (preg_match('_^unix/freebsd_', $pageview['OperatingSystem'])) $osIcon = 'freebsd';
						else if (preg_match('_^mac/i(pad|phone)/7_', $pageview['OperatingSystem'])) $osIcon = 'ios7';
						else if (preg_match('_^mac/i(pad|phone)/6_', $pageview['OperatingSystem'])) $osIcon = 'ios6';
						else if (preg_match('_^mac/i(pad|phone)/5_', $pageview['OperatingSystem'])) $osIcon = 'ios5';
						else if (preg_match('_^mac/i(pad|phone)/4_', $pageview['OperatingSystem'])) $osIcon = 'ios4';
						else if (preg_match('_^mac/i(pad|phone)/3_', $pageview['OperatingSystem'])) $osIcon = 'ios3';
						else if (preg_match('_^mac/i(pad|phone)/2_', $pageview['OperatingSystem'])) $osIcon = 'ios2';
						else if (preg_match('_^mac/i(pad|phone)/1_', $pageview['OperatingSystem'])) $osIcon = 'ios1';
						else if (preg_match('_^mac/i(pad|phone)_', $pageview['OperatingSystem'])) $osIcon = 'ios';
						else if (preg_match('_^unix/linux_', $pageview['OperatingSystem'])) $osIcon = 'linux';
						else if (preg_match('_^mac/x/_', $pageview['OperatingSystem'])) $osIcon = 'macosx';
						else if (preg_match('_^mac/_', $pageview['OperatingSystem'])) $osIcon = 'macos';
						else if (preg_match('_^palm_', $pageview['OperatingSystem'])) $osIcon = 'palm';
						else if (preg_match('_^unix/sun_', $pageview['OperatingSystem'])) $osIcon = 'solaris';
						else if (preg_match('_^win/nt/8_', $pageview['OperatingSystem'])) $osIcon = 'win8';
						else if (preg_match('_^win/nt/7_', $pageview['OperatingSystem'])) $osIcon = 'win7';
						else if (preg_match('_^win/nt/vista_', $pageview['OperatingSystem'])) $osIcon = 'winvista';
						else if (preg_match('_^win/nt/(xp|2003)_', $pageview['OperatingSystem'])) $osIcon = 'winxp';
						else if (preg_match('_^win/nt/2000_', $pageview['OperatingSystem'])) $osIcon = 'win2000';
						else if (preg_match('_^win/_', $pageview['OperatingSystem'])) $osIcon = 'win';
						else if (preg_match('_^java/me/_', $pageview['OperatingSystem'])) $osIcon = 'java';
						else if (preg_match('_^webos_', $pageview['OperatingSystem'])) $osIcon = 'webos';
		
						echo '<td class="nbr">';
						$browserName = $pageview['Browser'];
						$browserVersion = '';
						if (preg_match('_^(.*)/([0-9.]+)$_', $browserName, $m))
						{
							$browserName = $m[1];
							$browserVersion = ' ' . $m[2];
						}
						$browserName = unbtr('browser:' . $browserName, true);

						$osName = $pageview['OperatingSystem'];
						$osVersion = '';
						$osSuffix = '';
						if (preg_match('_^(mac/x(?:/.*)?)/(intel|ppc)$_', $osName, $m))   // Mac OS X special case
						{
							if ($m[2] == 'intel') $osSuffix = ' (Intel)';
							if ($m[2] == 'ppc') $osSuffix = ' (PowerPC)';
							$osName = $m[1];
						}
						if (preg_match('_^(win/phone(?:/.*)?)/(arm)$_', $osName, $m))   // Windows Phone special case
						{
							if ($m[2] == 'arm') $osSuffix = ' (ARM)';
							$osName = $m[1];
						}
						$osName = unbtr('os:' . $osName, true);   // First check with trailing number (might have a special translation)
						if (preg_match('_^(.*)/([0-9.]+)$_', $osName, $m))
						{
							$osName = $m[1];
							$osVersion = ' ' . $m[2];
						}
						$osName = unbtr('os:' . $osName, true);

						if ($browserIcon)
							$browserIcon = 'browser/' . $browserIcon;
						else
							$browserIcon = 'unknown';
						if ($osIcon)
							$osIcon = 'os/' . $osIcon;
						else
							$osIcon = 'unknown';

						echo '<div>';
						ShowIcon($browserIcon, $browserName . $browserVersion);
						ShowIcon($osIcon, $osName . $osVersion . $osSuffix);

						$features = '';
						if ($pageview['JavaScript'])
							$features .= GetIcon('features/javascript', 'JavaScript');
						if ($pageview['Cookies'] === '0')
							$features .= GetIcon('features/cookies0', 'No cookies');
						if ($pageview['Cookies'] === '1')
							$features .= GetIcon('features/cookies1', 'Cookies');
						if ($pageview['DNT'] === '0')
							$features .= GetIcon('features/dnt0', 'DNT=0');
						if ($pageview['DNT'] === '1')
							$features .= GetIcon('features/dnt1', 'DNT=1');
						if ($pageview['AdBlock'])
							$features .= GetIcon('features/adblock', 'AdBlock');
						if ($features)
							echo '<span class="icon-separator"></span>' . $features;

						$features = '';
						if ($pageview['Flash'])
							$features .= GetIcon('features/flash', 'Flash');
						if ($pageview['Java'])
							$features .= GetIcon('features/java', 'Java');
						if ($pageview['Silverlight'])
							$features .= GetIcon('features/silverlight', 'Silverlight');
						if ($pageview['WindowsMedia'])
							$features .= GetIcon('features/windowsmedia', 'Windows Media');
						if ($pageview['Quicktime'])
							$features .= GetIcon('features/quicktime', 'Quicktime');
						if ($features)
							echo '<span class="icon-separator"></span>' . $features;
						echo '</div>';
		
						echo '<div>';
						if ($pageview['Mobile'])
							ShowIcon('features/mobile', unbtrc('mobile device'));
						if (strlen($pageview['ScreenWidth']))
						{
							$dx = $pageview['ScreenAvailWidth'] - $pageview['ViewportWidth'];
							$dy = $pageview['ScreenAvailHeight'] - $pageview['ViewportHeight'];
							$fs = ($dx <= 25 && $dy <= 200) ? '_fullscreen' : '';
		
							echo '<span title="' . unbtrc('viewport size') . '">' . GetIcon('window' . $fs) .
								$pageview['ViewportWidth'] . '×' . $pageview['ViewportHeight'] . '</span> &nbsp; ' .
								'<span title="' . unbtrc('screen size') . ', ' . unbtr('available screen') . unbcol() . ' ' .
								$pageview['ScreenAvailWidth'] . '×' . $pageview['ScreenAvailHeight'] . '">' . GetIcon('screen') .
								$pageview['ScreenWidth'] . '×' . $pageview['ScreenHeight'] .
								'<small style="position: relative; top: -1px;">@</small>' . $pageview['ScreenColors'] . '</span>';
						}
						echo '</div>';

						echo '<div>';
						foreach (Tomo::GetConfVisitorVars() as $name => $type)
						{
							if (isset($pageview['vv_' . $name]))
							{
								ShowIcon('visitor_variable', unbtrc('custom visitor variable'));
								echo htmlspecialchars($name) . ': ' . htmlspecialchars($pageview['vv_' . $name]) . '<br>';
							}
						}
						foreach (Tomo::GetConfSessionVars() as $name => $type)
						{
							if (isset($pageview['sv_' . $name]))
							{
								ShowIcon('session_variable', unbtrc('custom session variable'));
								echo htmlspecialchars($name) . ': ' . htmlspecialchars($pageview['sv_' . $name]) . '<br>';
							}
						}
						echo '</div>';
						echo '</td>';
		
						echo '<td>';
						#echo $pageview['Domain'];
						$referrer = null;
						if ($pageview['ReferrerHost'])
						{
							$referrer = ($pageview['ReferrerSsl'] ? 'https://' : 'http://') .
								$pageview['ReferrerHost'] . $pageview['ReferrerUrlPath'];
						}
						$sourceKnown = false;
						if ($pageview['Campaign'])
						{
							ShowIcon('spread', unbtrc('campaign'));
							echo unbtrc('campaign') . unbcol() . ' <strong>' . htmlspecialchars($pageview['Campaign']) . '</strong><br>';
							$sourceKnown = true;
						}
						if ($pageview['SearchEngine'])
						{
							$seIcon = 'search';
							if (preg_match('_^ask_i', $pageview['SearchEngine'])) $seIcon = 'ask';
							else if (preg_match('_^bing_i', $pageview['SearchEngine'])) $seIcon = 'bing';
							else if (preg_match('_^google_i', $pageview['SearchEngine'])) $seIcon = 'google';
							else if (preg_match('_^web\.de_i', $pageview['SearchEngine'])) $seIcon = 'web_de';
							else if (preg_match('_^gmx_i', $pageview['SearchEngine'])) $seIcon = 'gmx';
							else if (preg_match('_^yahoo_i', $pageview['SearchEngine'])) $seIcon = 'yahoo';
							else if (preg_match('_^yandex\.ru_i', $pageview['SearchEngine'])) $seIcon = 'yandex';
							else if (preg_match('_^baidu_i', $pageview['SearchEngine'])) $seIcon = 'baidu';
							#else if (preg_match('_^scroogle_i', $pageview['SearchEngine'])) $seIcon = 'scroogle';
		
							$seUrl = '';
							if (preg_match('_^google images_i', $pageview['SearchEngine'])) $seUrl = 'http://www.google.com/images?q=';
							else if (preg_match('_^google videos_i', $pageview['SearchEngine'])) $seUrl = 'http://www.google.com/search?tbm=vid&q=';
							else if (preg_match('_^google news_i', $pageview['SearchEngine'])) $seUrl = 'http://www.google.com/search?tbm=nws&q=';
							else if (preg_match('_^google_i', $pageview['SearchEngine'])) $seUrl = 'http://www.google.com/search?q=';
							else if (preg_match('_^yahoo images_i', $pageview['SearchEngine'])) $seUrl = 'http://images.search.yahoo.com/search/images?p=';
							else if (preg_match('_^yahoo_i', $pageview['SearchEngine'])) $seUrl = 'http://search.yahoo.com/search?p=';
							else if (preg_match('_^bing images_i', $pageview['SearchEngine'])) $seUrl = 'http://www.bing.com/images/search?q=';
							else if (preg_match('_^bing_i', $pageview['SearchEngine'])) $seUrl = 'http://www.bing.com/search?q=';
							else if (preg_match('_^ask\.com_i', $pageview['SearchEngine'])) $seUrl = 'http://www.ask.com/web?q=';
							else if (preg_match('_^yandex\.ru_i', $pageview['SearchEngine'])) $seUrl = 'http://yandex.ru/yandsearch?text=';
							else if (preg_match('_^baidu_i', $pageview['SearchEngine'])) $seUrl = 'http://www.baidu.com/s?wd=';
							else if (preg_match('_^web\.de_i', $pageview['SearchEngine'])) $seUrl = 'http://suche.web.de/web?q=';
							else if (preg_match('_^gmx_i', $pageview['SearchEngine'])) $seUrl = 'http://suche.gmx.net/web?q=';
		
							if (preg_match('_^(google|yahoo)_i', $pageview['SearchEngine']))
							{
								// Use actual domain from referrer for our search link
								$seUrl = preg_replace('_^(https?://)[^/]+/_i', '$1' . $pageview['ReferrerHost'] . '/', $seUrl);
							}
		
							ShowIcon('search/' . $seIcon, unbtrc('search engine') . unbcol() . ' ' . $pageview['SearchEngine']);
							// Add search country flag
							#if (preg_match('_www\.google\.([a-z]{2})$|\b([a-z]{2})\.search\.yahoo\.com$_i', $pageview['ReferrerHost'], $m))
							#{
							#	ShowIcon('country/' . strtolower($m[1]));
							#}
							if ($seUrl)
								echo '<a href="' . htmlspecialchars($seUrl . urlencode($pageview['SearchKeywords'])) . '">';
							echo htmlspecialchars($pageview['SearchEngine']);
							if (strlen($pageview['SearchKeywords']))
							{
								echo ': <strong>' . htmlspecialchars($pageview['SearchKeywords']) . '</strong>';
								if ($seUrl)
									echo '</a>';
							}
							else
							{
								if ($seUrl)
									echo '</a>';
								echo ' – <em>' . unbtrc('no search terms available') . '</em>';
							}
							if ($pageview['SearchResultPage'])
							{
								echo ' &nbsp; <span title="' . unbtrc('page') . ' ' . $pageview['SearchResultPage'] . '">';
								ShowIcon('searchpage');
								echo $pageview['SearchResultPage'];
								echo '</span>';
							}
							if (strlen($pageview['SearchImageUrl']))
							{
								echo '<br>';
								ShowIcon('empty');
								ShowIcon('imgzoom', unbtrc('search image'));
								$urlPath = preg_replace('_^https?://.*?/_', '/', $pageview['SearchImageUrl']);
								echo '<a href="' . htmlspecialchars($pageview['SearchImageUrl']) . '">' . htmlspecialchars(str_limit($urlPath, 67)) . '</a>';
							}
							#echo ' (<a href="' . htmlspecialchars($referrer) . '">RawRef</a>)';
							echo '<br>';
							$sourceKnown = true;
						}
						if ($referrer && !$pageview['SearchEngine'])
						{
							if ($pageview['ReferrerHost'] == $pageview['Domain'])
							{
								ShowIcon('continuation', unbtrc('session continuation'));
								echo '<a href="' . htmlspecialchars($referrer) . '">' . htmlspecialchars(str_limit($pageview['ReferrerUrlPath'], 70)) . '</a>';
							}
							else
							{
								ShowIcon('link', unbtrc('referring website'));
								echo '<a href="' . htmlspecialchars($referrer) . '">' . htmlspecialchars(str_limit($referrer, 70)) . '</a>';
							}
							echo '<br>';
							$sourceKnown = true;
						}
						if (!$sourceKnown)
						{
							if (!$pageview['NewVisitor'])
							{
								ShowIcon('direct_bookmark', unbtrc('direct entry or bookmark'));
								echo unbtrc('direct entry or bookmark');
							}
							else
							{
								ShowIcon('direct', unbtrc('direct entry'));
								echo unbtrc('direct entry');
							}
							echo '<br>';
						}
						ShowIcon('sum');
						echo unbtrc('session hit count', $sessionHitCount) . ' ' . unbtr('session page count', $sessionPageCount);
						if (isset($sessionTime))
						{
							echo ' ' . unbtr('session time over') . ' ' .
								preg_replace('_^0(?!:)_', '', gmdate($sessionTime >= 3600 ? 'G:i:s' : 'i:s', $sessionTime));   // NOTE: Omits days if > 24 h
							$sessionDuration = intval($pageview['SessionDuration']);
							if ($sessionDuration > 0)
								echo ' / ' . preg_replace('_^0(?!:)_', '', gmdate($sessionDuration >= 3600 ? 'G:i:s' : 'i:s', $sessionDuration));   // NOTE: Omits days if > 24 h
						}
						echo '</td>';
		
						echo '</tr>';
					}
		
					if ($i == 0 || $pageview['PageviewId'] != $pageviews[$i - 1]['PageviewId'])
					{
						// New pageview
						$totalPageviewCount++;
						if ($i == 0) $totalFirstTime = UnbDatabase::ConvertDateTime($pageview['PageviewTime']);
						$totalLastTime = UnbDatabase::ConvertDateTime($pageview['PageviewTime']);
						$prevPageviewId = $pageviewId;
						$prevOriginalPageviewId = $originalPageviewId;
		
						$pageviewId = intval($pageview['PageviewId']);
						$originalPageviewId = intval($pageview['OriginalPageviewId']);
						
						// Determine whether this URL has been visited before in the same session
						$url = $pageview['UrlPath'];
						if ($pageview['UrlQuery'])
							$url .= '?' . $pageview['UrlQuery'];
						if (!in_array($url, $seenUrls))
						{
							$seenUrls[] = $url;
							$newUrlInSession = true;
						}
						else
						{
							$newUrlInSession = false;
						}

						// Detect longer breaks between the previous and current pageview
						$pageviewTime = UnbDatabase::ConvertDateTime($pageview['PageviewTime']);
						if ($pageview['ActivityTimespan'] > 0)
						{
							$pageviewUpdateTime = UnbDatabase::ConvertDateTime($pageview['PageviewUpdateTime']);
						}
						else
						{
							// Don't remember when this pageview has ended because we have no
							// reliable data about that.
							$pageviewUpdateTime = 0;
						}
						$isLater = $prevPageviewUpdateTime && $pageviewTime - $prevPageviewUpdateTime > 10;
						$prevPageviewUpdateTime = $pageviewUpdateTime;

						$cssClass = '';
						if (isset($pageview['ActivityTimespan']) && $pageview['ActivityTimespan'] < 8) $cssClass .= 'short-pageview ';
						else if ($pageview['CampaignRedirect'] || $pageview['Prerender']) $cssClass .= 'short-pageview ';
						if ($isLater) $cssClass .= 'later-pageview ';
						if ($cssClass) $cssClass = ' class="' . trim($cssClass) . '"';
		
						echo '<tr' . $cssClass . '>';
		
						echo '<td class="nbr" style="text-align: right;">';
						$diff = UnbDatabase::ConvertDateTime($pageview['PageviewTime']) - UnbDatabase::ConvertDateTime($prevPageviewTime);
						$prevPageviewTime = $pageview['PageviewTime'];
						#if ($diff)
						{
							echo '<span title="' . unbtm(UnbDatabase::ConvertDateTime($pageview['PageviewTime']), UnbTimeFormat::DateTimeSec) .
								' – ' . unbtm(UnbDatabase::ConvertDateTime($pageview['PageviewUpdateTime']), UnbTimeFormat::TimeSec) .
								', PageviewId: ' . $pageview['PageviewId'] . '">+' .
								preg_replace('_^0(?!:)_', '', gmdate($diff >= 3600 ? 'G:i:s' : 'i:s', $diff)) . '</span>';   // NOTE: Omits days if > 24 h
						}
						echo '</td>';
		
						echo '<td class="nbr">';
						echo '<span title="' . unbtrc('page load time') . unbcol() . ' ';
						if (isset($pageview['PageLoadTime']) && $pageview['PageLoadTime'] != -1)
							echo abs($pageview['PageLoadTime']) . ' ms';
						else
							echo unbtr('unknown');
						echo '" style="padding-left: 1px;">';   // timespan icon is 9px wide, action's timeinpage icon is 10px wide
						$timespan = '';
						if (isset($pageview['ActivityTimespan']))
						{
							$tsi = intval($pageview['ActivityTimespan']);
							$timespan = preg_replace('_^0(?!:)_', '', gmdate($tsi >= 3600 ? 'G:i:s' : 'i:s', $tsi));   // NOTE: Omits days if > 24 h
							if ($tsi > 10 * 60)
							{
								// Highlight very long activity timespans
								$timespan = '<span class="long-activity-timespan">' . $timespan . '</span>';
							}
						}
						if ($pageview['PageviewUpdateTime'] != $pageview['PageviewTime'])
						{
							$diff = UnbDatabase::ConvertDateTime($pageview['PageviewUpdateTime']) - UnbDatabase::ConvertDateTime($pageview['PageviewTime']);
							#if ($diff)
							{
								$timespan .= ($timespan ? ' / ' : '? / ') .
									preg_replace('_^0(?!:)_', '', gmdate($diff >= 3600 ? 'G:i:s' : 'i:s', $diff));   // NOTE: Omits days if > 24 h
							}
						}
						if ($timespan)
						{
							echo GetIcon('timespan') . $timespan . '&nbsp; &nbsp; ';
						}
						if (abs($pageview['PageLoadTime']) > 2000)
						{
							echo GetIcon('sand') . unbnum(abs($pageview['PageLoadTime'])) . ' ms&nbsp; &nbsp; ';
						}
						if ($pageview['PageLoadTime'] < 0)
						{
							echo GetIcon('canceled') . unbtrc('loading canceled');
						}
						echo '</span>';
						echo '</td>';
		
						echo '<td>';
						foreach (Tomo::GetConfPageviewVars() as $name => $type)
						{
							if (isset($pageview['pv_' . $name]))
							{
								ShowIcon('pageview_variable', unbtrc('custom pageview variable'));
								echo htmlspecialchars($name) . ': ' . htmlspecialchars($pageview['pv_' . $name]) . '<br>';
							}
						}
						echo '</td>';

						echo '<td>';
						$isReload =
							$originalPageviewId > 0 && $originalPageviewId == $prevPageviewId ||
							// This is a re-view of the previous pageview, so it must be a reload.
							$originalPageviewId > 0 && $originalPageviewId == $prevOriginalPageviewId;
							// This is a re-view of the same pageview as the previous pageview is, so it
							// must be a reload.
						$isBackNav =
							$originalPageviewId > 0 && $prevOriginalPageviewId == 0 ||
							// The previous pageview was no backward navigation and this is a re-view of a
							// page, so it must be a backward navigation.
							$originalPageviewId > 0 && $originalPageviewId < $prevOriginalPageviewId;
							// This page is a re-view of an earlier page than the one before, so it must be
							// a backward navigation.
						$isForwardNav =
							$prevOriginalPageviewId > 0 && $originalPageviewId > $prevOriginalPageviewId;
							// The previous pageview was a re-view of a page and this page is a re-view of
							// a later page than the one before, so it must be a forward navigation.

						if ($isReload && $prevPageviewOutlink)
						{
							// What looks like a reload is more likely to be a back navigation if
							// there was an outlink in between.
							$isReload = false;
							$isBackNav = true;
						}
						$prevPageviewOutlink = false;

						if ($pageview['EntryPage'] && $pageview['ExitPage'])
							ShowIcon('bounce', unbtrc('bounce'));
						else if ($pageview['EntryPage'])
							ShowIcon('entry', unbtrc('entry page'));
						#else if ($pageview['ExitPage'])
						#	ShowIcon('exit', unbtrc('exit page'));
						else if ($isReload && !$isLater)
							ShowIcon('navreload', unbtrc('reloaded page') . ', OriginalPageviewId: ' . $originalPageviewId);
						else if ($isBackNav && !$isReload)
							ShowIcon('navback', unbtrc('navigated back') . ', OriginalPageviewId: ' . $originalPageviewId);
						else if ($isForwardNav && !$isReload)
							ShowIcon('navforward', unbtrc('navigated forward') . ', OriginalPageviewId: ' . $originalPageviewId);
						else if (!$newUrlInSession)
							ShowIcon('navrevisit', unbtrc('revisited page'));
						else
							ShowIcon('empty');
						
						if ($pageview['Prerender'])
							echo '<s>';
						echo '<span title="' . htmlspecialchars($pageview['PageTitle']) . '">';
						$url = $pageview['UrlPath'];
						if ($pageview['UrlQuery'])
							$url .= '?' . $pageview['UrlQuery'];
						if ($pageview['UrlHash'])
							$url .= '#' . $pageview['UrlHash'];
						$linkUrl = ($pageview['Ssl'] ? 'https://' : 'http://') . $pageview['Domain'] . $url;
						echo '<a href="' . htmlspecialchars($linkUrl) . '" class="nocolor">';
						echo htmlspecialchars(str_limit($url, 65));
						echo '</a>';
						echo '</span>';
						if ($pageview['Prerender'])
							echo '</s>';

						if ($pageview['Ssl'])
							echo '&nbsp; ' . GetIcon('ssl', 'SSL', 'smicon');
						if ($pageview['PageLanguage'])
							echo '<span title="' . unbtrc('language:' . strtolower($pageview['PageLanguage'])) . '" style="margin-left: 1em;">' .
								GetIcon('language') . $pageview['PageLanguage'] .
								'</span>';
						if ($pageview['JavaScript'] && !$pageview['JavaScriptData'])
							echo ' [NoJs]';
						if (!$pageview['JavaScript'] && $pageview['JavaScriptData'])
							echo ' [Js]';
						if ($pageview['CampaignRedirect'])
							echo ' [CR]';
						echo '</td>';
		
						echo '</tr>';
					}
		
					if ($pageview['ActionId'])
					{
						// Action
						$cssClass = '';
						if ($pageview['ActionType']) $cssClass .= 'action ';
						if ($cssClass) $cssClass = ' class="' . trim($cssClass) . '"';
		
						echo '<tr' . $cssClass . '>';
		
						echo '<td></td>';
		
						echo '<td class="nbr">';
						$diff = UnbDatabase::ConvertDateTime($pageview['ActionTime']) - UnbDatabase::ConvertDateTime($prevPageviewTime);
						#if ($diff)
						{
							echo '<span title="' . unbtrc('time since page load') . '">';
							ShowIcon('timeinpage');
							echo '</span>';
							echo '<span title="' . unbtm(UnbDatabase::ConvertDateTime($pageview['ActionTime']), UnbTimeFormat::DateTimeSec) . '">' .
								preg_replace('_^0(?!:)_', '', gmdate($diff >= 3600 ? 'G:i:s' : 'i:s', $diff)) . '</span>';   // NOTE: Omits days if > 24 h
						}
						echo '</td>';
		
						echo '<td></td>';
		
						echo '<td>';
						ShowIcon('empty');
						if ($pageview['ActionType'] == Tomo::action_Outlink)
						{
							$prevPageviewOutlink = true;

							ShowIcon('outlink', unbtrc('outlink'));
							echo '<a href="' . htmlspecialchars($pageview['ActionUrl']) . '">' . htmlspecialchars(str_limit($pageview['ActionUrl'], 67)) . '</a>';
						}
						else if ($pageview['ActionType'] == Tomo::action_Download)
						{
							$contentType = ContentType::GetContentTypeForFileName($pageview['ActionUrl']);
							ShowIcon('ctype/' . ContentType::ContentTypeIconSprite($contentType), unbtrc('download') . unbcol() . ' ' . ContentType::GetContentTypeDescription($contentType));
							$urlPath = preg_replace('_^https?://.*?/_', '/', $pageview['ActionUrl']);
							echo '<span title="' . htmlspecialchars($pageview['ActionUrl']) . '">' . htmlspecialchars(str_limit($urlPath, 67)) . '</span>';
						}
						else if ($pageview['ActionType'] == Tomo::action_ImgZoom)
						{
							ShowIcon('imgzoom', unbtrc('image zoom'));
							$urlPath = preg_replace('_^https?://.*?/_', '/', $pageview['ActionUrl']);
							echo '<a href="' . htmlspecialchars($pageview['ActionUrl']) . '">' . htmlspecialchars(str_limit($urlPath, 67)) . '</a>';
						}
						else if ($pageview['ActionType'] == Tomo::action_SectionExpand)
						{
							ShowIcon('expand', unbtrc('section expand'));
							echo '<span>' . htmlspecialchars($pageview['ActionData']) . '</span>';
						}
						else if ($pageview['ActionType'] == Tomo::action_SiteSearch)
						{
							ShowIcon('search/search', unbtrc('site search'));
							echo '<span>' . htmlspecialchars($pageview['ActionData']) . '</span>';
						}
						else
						{
							$confActions = Tomo::GetConfActions();
							$validAction = false;
							foreach ($confActions as $name => $type)
							{
								if ($pageview['ActionType'] == $type)
								{
									$validAction = true;
									ShowIcon('empty');   // TODO: Use better icon
									echo $name;
									if (isset($pageview['ActionUrl']))
									{
										echo ': <a href="' . htmlspecialchars($pageview['ActionUrl']) . '">' . htmlspecialchars(str_limit($pageview['ActionUrl'], 50)) . '</a>';
									}
									if (isset($pageview['ActionData']))
									{
										echo isset($pageview['ActionUrl']) ? ', ' : ': ';
										echo '<span title="' . htmlspecialchars($pageview['ActionData']) . '">' . htmlspecialchars(str_limit($pageview['ActionData'], 50)) . '</span>';
									}
									break;
								}
							}
							if (!$validAction)
							{
								echo '<em>Invalid action type ' . $pageview['ActionType'] . '</em>';
							}
						}
						echo '</td>';
		
						echo '</tr>';
					}
				}
				echo '</table>';
				
				if (count($pageviews) > $limit)
				{
					echo '<p><em>The output was limited to ' . unbnum($limit) . ' records. More records are available.</em></p>';
				}
				
				echo '<table class="with-header" style="margin-top: 2em;">';
				echo '<tr><th colspan="2">' . unbtrc('summary') . '</th></tr>';
				echo '<tr><td>' . unbtrc('summary.unique visitors') . unbcol() . '</td><td>' . unbnum(count($totalVisitors)) . '</td></tr>';
				echo '<tr><td>' . unbtrc('sessions') . unbcol() . '</td><td>' . unbnum($totalSessionCount) . '</td></tr>';
				echo '<tr><td>' . unbtrc('pageviews') . unbcol() . '</td><td>' . unbnum($totalPageviewCount) . '</td></tr>';
				echo '<tr><td>' . unbtrc('summary.first') . unbcol() . '</td><td>' .
					($totalFirstTime ? unbtm($totalFirstTime, UnbTimeFormat::DateTimeSec) : '–') .
					'</td></tr>';
				echo '<tr><td>' . unbtrc('summary.last') . unbcol() . '</td><td>' .
					($totalLastTime ? unbtm($totalLastTime, UnbTimeFormat::DateTimeSec) : '–') .
					'</td></tr>';
				echo '</table>';
			} // visitor log view
		
			// ————————————————————————————————————————— QUERY VIEW —————————————————————————————————————————
		
			else if ($viewType == 'query')
			{
				echo '<a href="?view=index" title="' . unbtrc('overview link') . '" class="roundnavlink">' . GetIcon('roundback') . '</a>';
		
				// Read all XML files into categories and titles
				$categories = array();
				foreach (glob(self::$queriesPath . '*.xml') as $filename)
				{
					$value = basename($filename, '.xml');
					$xquery = simplexml_load_file(self::$queriesPath . $value . '.xml');
					$title = (string) $xquery->title;
					$category = (string) $xquery->category;
		
					if (!isset($categories[$category]))
					{
						$categories[$category] = array();
					}
					$categories[$category][$title] = $value;
				}
		
				// Show query selection form
				echo '<form style="float: right; margin-top: 2px; margin-right: 8px;">';
				echo '<input type="hidden" name="view" value="query">';
				echo '<select name="name" onchange="this.form.submit()">';
				echo '<option value="">' . unbtrc('select query') . '</option>';
				ksort($categories);
				foreach ($categories as $category => $titles)
				{
					$lines = str_repeat('—', round((35 - strlen($category)) / 3.5));
					echo '<option disabled value="">' . $lines . ' ' . htmlspecialchars($category) . ' ' . $lines . '</option>';
		
					ksort($titles);
					foreach ($titles as $title => $name)
					{
						echo '<option value="' . htmlspecialchars($name) . '"' . ($_GET['name'] == $name ? ' selected' : '') . '>' .
							htmlspecialchars($title) . '</option>';
					}
				}
				echo '</select><noscript> <input type="submit" value="' . unbtrc('view button') . '"></noscript></form>';
		
				$filename = $_GET['name'];
		
				if ($filename)
				{
					$title = '';
					$withPageviews = false;
					$withActions = false;
					$groupMaxLevel = 0;
					$columns = array();
					$limits = array();
		
					if (preg_match('_^[a-z0-9-]+$_', $filename) && file_exists(self::$queriesPath . $filename . '.xml'))
					{
						$xquery = simplexml_load_file(self::$queriesPath . $filename . '.xml');
		
						$title = (string) $xquery->title;
						$withPageviews = (string) $xquery->withpageviews == 'true';
						$withActions = (string) $xquery->withactions == 'true';
						if ($xquery->groupmax)
							$groupMaxLevel = (int) $xquery->groupmax;
		
						foreach ($xquery->columns->column as $xcol)
						{
							$col = array();
							if ($xcol->source)
								$col['source'] = (string) $xcol->source;
							if ($xcol->source && $xcol->source->attributes()->function)
								$col['function'] = (string) $xcol->source->attributes()->function == 'true';
							if ($xcol->alias)
								$col['alias'] = (string) $xcol->alias;
							else
								$col['alias'] = $col['source'];
							if ($xcol->barsource)
								$col['barsource'] = (string) $xcol->barsource;
							if ($xcol->title)
								$col['title'] = (string) $xcol->title;
							if ($xcol->title && $xcol->title->attributes()->colspan)
								$col['titlecolspan'] = (int) $xcol->title->attributes()->colspan;
							if ($xcol->empty)
								$col['empty'] = (string) $xcol->empty;
							if ($xcol->null)
								$col['null'] = (string) $xcol->null;
							if ($xcol->group)
								$col['group'] = (int) $xcol->group;
							if ($xcol->sort)
								$col['sortindex'] = (int) $xcol->sort;
							if ($xcol->sort && $xcol->sort->attributes()->dir)
								$col['sortdir'] = (string) $xcol->sort->attributes()->dir == 'desc' ? 'desc' : '';
							if ($xcol->resolve)
								$col['resolve'] = (string) $xcol->resolve;
							if ($xcol->set)
								$col['set'] = (string) $xcol->set == 'true';
							if ($xcol->total)
								$col['total'] = (string) $xcol->total;
							if ($xcol->format)
								$col['format'] = (string) $xcol->format;
							if ($xcol->totalformat)
								$col['totalformat'] = (string) $xcol->totalformat;
							if ($xcol->align)
								$col['align'] = (string) $xcol->align;
							if ($xcol->width)
								$col['width'] = (int) $xcol->width;
							if ($xcol->color)
								$col['color'] = (string) $xcol->color;
							if ($xcol->percent)
								$col['percent'] = (string) $xcol->percent == 'true';
							if ($xcol->absolute)
								$col['absolute'] = true;
							if ($xcol->description)
								$col['description'] = (string) $xcol->description;
							if ($xcol->fill)
								$col['fill'] = (string) $xcol->fill;
							if ($xcol->default)
								$col['default'] = (string) $xcol->default;   // required for fill
							if ($xcol->filters)
							{
								if ($xcol->filters->attributes()->all)
									$col['allfilters'] = (string) $xcol->filters->attributes()->all == 'true';
								$filters = array();
								foreach ($xcol->filters->filter as $xfilter)
								{
									$filter = array();
									if ($xfilter->attributes()->not)
										$filter['not'] = (string) $xfilter->attributes()->not == 'true';
									if ($xfilter->attributes()->op)
										$filter['op'] = (string) $xfilter->attributes()->op;
									if ($xfilter->attributes()->value)
										$filter['value'] = (string) $xfilter->attributes()->value;
									if ($xfilter->attributes()->value2)
										$filter['value2'] = (string) $xfilter->attributes()->value2;
									if ($xfilter->attributes()->int && (string) $xfilter->attributes()->int == 'true')
									{
										$filter['value'] = intval($filter['value']);
										$filter['value2'] = intval($filter['value2']);
									}
		
									$filters[] = $filter;
								}
								$col['filters'] = $filters;
							}
							if ($xcol->pgfilters)
							{
								if ($xcol->pgfilters->attributes()->all)
									$col['allpgfilters'] = (string) $xcol->pgfilters->attributes()->all == 'true';
								$pgfilters = array();
								foreach ($xcol->pgfilters->filter as $xfilter)
								{
									$filter = array();
									if ($xfilter->attributes()->not)
										$filter['not'] = (string) $xfilter->attributes()->not == 'true';
									if ($xfilter->attributes()->op)
										$filter['op'] = (string) $xfilter->attributes()->op;
									if ($xfilter->attributes()->value)
										$filter['value'] = (string) $xfilter->attributes()->value;
									if ($xfilter->attributes()->value2)
										$filter['value2'] = (string) $xfilter->attributes()->value2;
									if ($xfilter->attributes()->int && (string) $xfilter->attributes()->int == 'true')
									{
										$filter['value'] = intval($filter['value']);
										$filter['value2'] = intval($filter['value2']);
									}
		
									$pgfilters[] = $filter;
								}
								$col['pgfilters'] = $pgfilters;
							}
		
							if (isset($col['source']))
							{
								$firstTime = UnbDatabase::ConvertDateTime($tomo->GetFirstSessionTime());
								$daysCount = (time() - $firstTime) / 86400;
		
								$col['source'] = preg_replace('_\$daysCount_i', $daysCount, $col['source']);
							}
		
							$columns[] = $col;
						}
		
						if ($xquery->limits) foreach ($xquery->limits->limit as $xlim)
						{
							if ($xlim->attributes()->group)
								$limits[intval($xlim->attributes()->group) - 1] = (int) $xlim;
						}
					}
		
					#echo '<pre>';
					#print_r($columns);
					#echo '</pre>';
					#exit();
		
					echo '<h1>' . htmlspecialchars($title) . ' <span class="domain">tòmò@' . $_SERVER['SERVER_NAME'] . '</span></h1>';
		
					echo '</div>'; // #page-header

					echo '<div id="page-content">';

					$records = $tomo->GetRecords($withPageviews, $withActions, $columns, $limits);
					
					if (count($records))
					{
						// Fill in missing values
						$prevValues = array();
						$utc = new DateTimeZone('UTC');
						$afterLastRecord = false;
						
						// Include fake record after the last record to fill values at the end of the range (if known)
						for ($i = 0; $i <= count($records); $i++)
						{
							if ($i < count($records))
							{
								$rec = $records[$i];
							}
							else
							{
								$rec = array();
								$lastRecord = $records[count($records) - 1];
								if ($lastRecord['_group']) $rec['_group'] = $lastRecord['_group'];
								$afterLastRecord = true;
							}
							
							foreach ($columns as $col)
							{
								if (strlen($col['alias']))
								{
									$value = $rec[$col['alias']];
									$prevValue = $prevValues[$col['alias']];

									// Fill in missing months in the range or between the first and last record
									if (preg_match('/^month:([0-9-]+)\.\.([0-9-]+)$/', $col['fill'], $m) ||
										$col['fill'] == 'month' && isset($prevValue) && !$afterLastRecord)
									{
										$startValue = $m[1];
										$endValue = $m[2];
										$dtEnd = null;
										if ($m)
										{
											$dtEnd = new DateTime($endValue, $utc);
											$dtNow = new DateTime('now');
											$dtNow->setTimeZone($utc);
											if ($dtEnd > $dtNow)
											{
												// Don't go beyond today
												$dtEnd = $dtNow;
											}
										}
										if (!isset($prevValue))
										{
											if ($m)
											{
												$prevValue = $startValue;
												$dtPrev = new DateTime($prevValue, $utc);
											}
										}
										else
										{
											$dtPrev = new DateTime($prevValue, $utc);
											$dtPrev->add(new DateInterval('P1M'));
										}
										$dtValue = null;
										if (!$afterLastRecord)
										{
											$dtValue = new DateTime($value, $utc);
										}
										
										while (!$afterLastRecord && $dtPrev < $dtValue && (!isset($dtEnd) || $dtPrev <= $dtEnd) ||
											$afterLastRecord && $dtPrev <= $dtEnd)
										{
											// Missing next day, insert it
											
											$newRec = array();
											foreach ($columns as $newCol)
											{
												$newRec[$newCol['alias']] = $newCol['default'];
											}
											$newRec[$col['alias']] = $dtPrev->format('Y-m');
											if ($rec['_group']) $newRec['_group'] = $rec['_group'];
											array_splice($records, $i, 0, array($newRec));
											$i++;
											$dtPrev->add(new DateInterval('P1M'));
										}
									}

									// Fill in missing days in the range or between the first and last record
									if (preg_match('/^day:([0-9-]+)\.\.([0-9-]+)$/', $col['fill'], $m) ||
										$col['fill'] == 'day' && isset($prevValue) && !$afterLastRecord)
									{
										$startValue = $m[1];
										$endValue = $m[2];
										$dtEnd = null;
										if ($m)
										{
											$dtEnd = new DateTime($endValue, $utc);
											$dtNow = new DateTime('now');
											$dtNow->setTimeZone($utc);
											if ($dtEnd > $dtNow)
											{
												// Don't go beyond today
												$dtEnd = $dtNow;
											}
										}
										if (!isset($prevValue))
										{
											if ($m)
											{
												$prevValue = $startValue;
												$dtPrev = new DateTime($prevValue, $utc);
											}
										}
										else
										{
											$dtPrev = new DateTime($prevValue, $utc);
											$dtPrev->add(new DateInterval('P1D'));
										}
										$dtValue = null;
										if (!$afterLastRecord)
										{
											$dtValue = new DateTime($value, $utc);
										}
										
										while (!$afterLastRecord && $dtPrev < $dtValue && (!isset($dtEnd) || $dtPrev <= $dtEnd) ||
											$afterLastRecord && $dtPrev <= $dtEnd)
										{
											// Missing next day, insert it
											
											$newRec = array();
											foreach ($columns as $newCol)
											{
												$newRec[$newCol['alias']] = $newCol['default'];
											}
											$newRec[$col['alias']] = $dtPrev->format('Y-m-d');
											if ($rec['_group']) $newRec['_group'] = $rec['_group'];
											array_splice($records, $i, 0, array($newRec));
											$i++;
											$dtPrev->add(new DateInterval('P1D'));
										}
									}

									// Fill in missing integer values in the range
									// Ex.: 10..40
									if (preg_match('/^([0-9]+)\.\.([0-9]+)$/', $col['fill'], $m))
									{
										$startValue = intval($m[1]);
										$endValue = intval($m[2]);
										if (!isset($prevValue)) $prevValue = $startValue - 1;
										
										$prevValue++;
										if ($prevValue < $startValue) $prevValue = $startValue;
										while ($prevValue < $value && $prevValue <= $endValue ||
											$afterLastRecord && $prevValue <= $endValue)
										{
											// Missing next integer in range, insert it
											
											$newRec = array();
											foreach ($columns as $newCol)
											{
												$newRec[$newCol['alias']] = $newCol['default'];
											}
											$newRec[$col['alias']] = $prevValue;
											if ($rec['_group']) $newRec['_group'] = $rec['_group'];
											array_splice($records, $i, 0, array($newRec));
											$i++;
											$prevValue++;
										}
									}

									// Fill in missing values from the list
									// Ex.: valueA;valueB;valueC
									if (preg_match('/^.*?(;.*?)+$/', $col['fill'], $m))
									{
										$values = explode(';', $col['fill']);
										if (!isset($prevValue))
										{
											// Expect first value from list
											$vi = 0;
											$prevValue = $values[$vi];
										}
										else
										{
											// Find next value from list to expect
											$vi = array_search($prevValue, $values);
											if ($vi !== false)
											{
												$vi++;
												$prevValue = $values[$vi];
											}
											else
											{
												// Current value not in list, accept it
												$prevValue = $value;
											}
										}
										while (isset($prevValue) &&
											($prevValue != $value ||
											$afterLastRecord && $vi < count($values)))
										{
											// Missing next value in list, insert it
											
											$newRec = array();
											foreach ($columns as $newCol)
											{
												$newRec[$newCol['alias']] = $newCol['default'];
											}
											$newRec[$col['alias']] = $prevValue;
											if ($rec['_group']) $newRec['_group'] = $rec['_group'];
											array_splice($records, $i, 0, array($newRec));

											$i++;
											$vi++;
											$prevValue = $values[$vi];
										}
									}

									$prevValues[$col['alias']] = $value;
								}
							}
						}
						
						echo '<table class="with-header vborder">';
		
						// Show column headers
						$maxGroup = 0;
						$remainingColspan = 0;
						echo '<tr>';
						foreach ($columns as $col)
						{
							$maxGroup = max($maxGroup, $col['group']);
							if (isset($col['title']))
							{
								if ($remainingColspan > 0)
								{
									$remainingColspan--;
									continue;
								}
		
								$cssClass = '';
								if (!strlen($col['title'])) $cssClass .= 'no-vborder ';
								if ($cssClass) $cssClass = ' class="' . trim($cssClass) . '"';
		
								$style = '';
								if ($col['align'] == 'center' && $col['titlecolspan'] <= 1) $style .= 'text-align: center; ';
								if ($col['align'] == 'right' && $col['titlecolspan'] <= 1) $style .= 'text-align: right; ';
								if ($style) $style = ' style="' . trim($style) . '"';
		
								$colspan = '';
								if ($col['titlecolspan'] > 1)
								{
									$colspan = ' colspan="' . $col['titlecolspan'] . '"';
									$remainingColspan = $col['titlecolspan'] - 1;
								}
								
								$title = '';
								if (strlen($col['description']) > 0)
								{
									$title = ' title="' . htmlspecialchars($col['description']) . '"';
								}
		
								echo '<th' . $cssClass . $style . $colspan . $title . '>';
								$title = htmlspecialchars($col['title']);
								$title = str_replace('-~', '-<br>', $title);
								$title = str_replace('~', '<br>', $title);
								echo $title;
								if ($col['sortindex'])
								{
									if ($col['sortdir'] == 'desc')
									{
										echo '<span class="sortsym">&#x' . ($col['sortindex'] > 1 ? '25bd' : '25bc') . ';</span>';
									}
									else
									{
										echo '<span class="sortsym">&#x' . ($col['sortindex'] > 1 ? '25b3' : '25b2') . ';</span>';
									}
								}
								echo '</th>';
							}
						}
						echo '</tr>';

						// Generate column max values and sums
						$sums = array();
						$counts = array();
						$max = array();
		
						foreach ($records as $rec)
						{
							foreach ($columns as $col)
							{
								if (isset($col['title']) &&
									strlen($col['alias']))
								{
									$value = $rec[$col['alias']];
									if (strlen($value) &&   // Don't sum unset values
										is_numeric($value))
									{
										$max[$col['alias']] = max($max[$col['alias']], floatval($value));
										if ($rec['_group'] <= 1)   // Don't sum up sub-group values, they're duplicates
										{
											$sums[$col['alias']] += floatval($value);
										}
									}
								}
							}
						}
		
						// Show all records
						foreach ($records as $recIndex => $rec)
						{
							$cssClass = '';
							if ($maxGroup && $rec['_group'] < $maxGroup) $cssClass .= 'group_' . min($rec['_group'], 4) . 'of' . min($maxGroup, 5) . ' ';
							if ($cssClass) $cssClass = ' class="' . trim($cssClass) . '"';
		
							$style = '';
							if ($style) $style = ' style="' . trim($style) . '"';
		
							echo '<tr' . $cssClass . $style . '>';
							foreach ($columns as $col)
							{
								if (isset($col['title']))
								{
									$cssClass = '';
									if ($col['group'] && $col['group'] < $rec['_group']) $cssClass .= 'same-group ';
									if (!strlen($col['title'])) $cssClass .= 'no-vborder ';
									if ($cssClass) $cssClass = ' class="' . trim($cssClass) . '"';
		
									$style = '';
									if ($col['align'] == 'center') $style .= 'text-align: center; ';
									if ($col['align'] == 'right') $style .= 'text-align: right; ';
									if ($style) $style = ' style="' . trim($style) . '"';
		
									echo '<td' . $cssClass . $style . '>';
									if (!$col['group'] || $col['group'] == $rec['_group'])
									{
										if (strlen($col['alias']))
										{
											$value = $rec[$col['alias']];
											if (strlen($value))
											{
												if ($col['resolve'] == 'country')
												{
													$country = unbtr('country:' . $value);
													if (!isset($country))
													{
														$country = GeoIP::GetCountryName($value);
													}
													if (isset($country))
													{
														$value = GetIcon('country/' . strtolower($value)) . ' ' . $country . ' [' . $value . ']';
													}
												}
												if ($col['resolve'] == 'language')
												{
													$language = unbtrc('language:' . $value);
													if (isset($language))
													{
														$value = $language . ' [' . $value . ']';
													}
												}
		
												$value = FormatValue($col['format'], $value);
												echo $value;
											}
											else if (isset($value) && strlen($col['empty']))
											{
												echo '<em>' . htmlspecialchars($col['empty']) . '</em>';
											}
											else if (!isset($value) && strlen($col['null']))
											{
												echo '<em>' . htmlspecialchars($col['null']) . '</em>';
											}
											else if ($value === '')
											{
												#echo '<em>(leer)</em>';
											}
											else if ($value === null)
											{
												echo '<em>(NULL)</em>';
											}
										}
										else if (strlen($col['barsource']))
										{
											if ($groupMaxLevel > 0 && $rec['_group'] == $groupMaxLevel)
											{
												// Starting a new group on the level that maximum values are limited to.
												// Update all $max and $sums values for this group.
												$max[$col['barsource']] = 0;
												$sums[$col['barsource']] = 0;
												
												for ($i = $recIndex + 1; $i < count($records); $i++)
												{
													$rec2 = $records[$i];
													if ($rec2['_group'] <= $groupMaxLevel) break;   // Stop at end of local group
													
													$value = floatval($rec2[$col['barsource']]);
													if (strlen($value) &&   // Don't sum unset values
														is_numeric($value))
													{
														$max[$col['barsource']] = max($max[$col['barsource']], floatval($value));
														$sums[$col['barsource']] += floatval($value);
													}
												}
												
												// Don't show a bar on this level
											}
											else if ($groupMaxLevel == 0 || $rec['_group'] > $groupMaxLevel)
											{
												$value = floatval($rec[$col['barsource']]);
												$maxWidth = $col['width'];
												$outerBar = false;
												if ($col['absolute'])
												{
													$width = $value * $maxWidth;
													$outerBar = true;
													#$marginRight = $maxWidth - $width;
												}
												else
												{
													$maxValue = $max[$col['barsource']];
													if ($maxValue > 0)
													{
														$width = $value / $maxValue * $maxWidth;
													}
													#$marginRight = 0;
												}

												if ($width >= 0)
												{
													if ($outerBar)
													{
														$style = 'width: ' . round($maxWidth, 2) . 'pt;';
														if (strlen($col['color']))
														{
															$col2 = BlendColors('white', 0.13, $col['color']);
															$style .= ' background-color: ' . $col2 . ';';
															$style .= ' background: linear-gradient(to bottom, ' . BlendColors('black', 0.96, $col2) . ', ' . $col2 . ');';
														}
														if ($rec['_group'] - $groupMaxLevel == 2) $style .= ' opacity: 0.6;';
														else if ($rec['_group'] - $groupMaxLevel == 3) $style .= ' opacity: 0.4;';
														else if ($rec['_group'] - $groupMaxLevel > 3) $style .= ' opacity: 0.3;';

														echo '<span class="backbar" style="' . $style . '">';
													}

													$style = 'width: ' . round($width, 2) . 'pt;';
													if (strlen($col['color']))
													{
														$style .= ' background-color: ' . $col['color'] . ';';
														$style .= ' background: linear-gradient(to bottom, ' . BlendColors('white', 0.7, $col['color']) . ', ' . $col['color'] . ');';
													}
													if ($rec['_group'] - $groupMaxLevel == 2) $style .= ' opacity: 0.6;';
													else if ($rec['_group'] - $groupMaxLevel == 3) $style .= ' opacity: 0.4;';
													else if ($rec['_group'] - $groupMaxLevel > 3) $style .= ' opacity: 0.3;';
													if ($value > 0) $style .= ' min-width: 1px;';
													#if ($marginRight != 0) $style .= ' margin-right: ' . round($marginRight, 2) . 'pt;';

													#echo '<span class="bar" style="' . $style . '">&nbsp;</span>';

													#if ($col['percent'])
													#{
													#	echo '<span class="bar-percent">' . FormatValue('%.1p', $value / $sums[$col['barsource']]) . '</span>';
													#}

													if ($sums[$col['barsource']] != 0)
													{
														$v = FormatValue('%.1p', $value / $sums[$col['barsource']]);
													}
													else
													{
														$v = 'Inf';
													}

													echo '<span class="bar" style="' . $style . '">';
													if ($col['percent'] && $width >= 30)
													{
														echo '<span class="bar-percent">' . $v . '</span>';
													}
													else
													{
														echo '&nbsp;';
													}
													echo '</span>';
													if ($col['percent'] && $width < 30)
													{
														echo '<span class="bar-percent">' . $v . '</span>';
													}

													if ($outerBar)
													{
														echo '</span>'; // .backbar
													}
												}
											}
										}
									}
									echo '</td>';
		
									if (strlen($value) &&   // Don't sum unset values
										$rec['_group'] <= 1 && $col['total'])   // Don't sum up sub-group values, they're duplicates
									{
										#$sums[$col['alias']] += $rec[$col['alias']];
										$counts[$col['alias']]++;
									}
								}
							}
							echo '</tr>';
						}
		
						// Show summary row
						if (count($counts))
						{
							echo '<tr class="summary">';
							foreach ($columns as $col)
							{
								if (isset($col['title']))
								{
									$cssClass = '';
									if (!strlen($col['title'])) $cssClass .= 'no-vborder ';
									if ($cssClass) $cssClass = ' class="' . trim($cssClass) . '"';
		
									$style = '';
									if ($col['align'] == 'center') $style .= 'text-align: center; ';
									if ($col['align'] == 'right') $style .= 'text-align: right; ';
									if ($style) $style = ' style="' . trim($style) . '"';
		
									echo '<td' . $cssClass . $style . '>';
		
									$value = null;
									$sign = '';
									if ($col['total'] == 'sum')
									{
										$value = $sums[$col['alias']];
										#$sign = '<small>&#x2211;</small> ';
										$sign = '<small>&#x3a3;</small> ';
									}
									else if ($col['total'] == 'avg')
									{
										if ($counts[$col['alias']])
											$value = $sums[$col['alias']] / $counts[$col['alias']];
										else
											$value = 0;
										$sign = '<small>&#x2205;</small> ';
										#$sign = '<small>Ø</small> ';
									}
		
									if (isset($value))
									{
										$format = $col['totalformat'];
										if (!strlen($format))
										{
											$format = $col['format'];
										}
										$value = FormatValue($format, $value);
										echo $sign . $value;
									}
									echo '</td>';
								}
							}
							echo '</tr>';
						}
		
						echo '</table>';
					}
					else
					{
						echo '<p><em>' . unbtr('query returned no data') . '</em></p>';
					}
				}
				else // !$filename
				{
					echo '<h1>' . unbtr('query view') . ' <span class="domain">tòmò@' . $_SERVER['SERVER_NAME'] . '</span></h1>';
					
					echo '</div>'; // #page-header

					echo '<div id="page-content">';

					echo '<p>' . unbtr('query view intro') . '</p>';

					echo '<div style="-moz-column-count: 3;">';
					foreach ($categories as $category => $titles)
					{
						echo '<h2>' . htmlspecialchars($category) . '</h2>';

						ksort($titles);
						echo '<ul>';
						foreach ($titles as $title => $name)
						{
							echo '<li><a href="?view=query&amp;name=' . htmlspecialchars($name) . '" class="novis">' .
								htmlspecialchars($title) . '</a></li>';
						}
						echo '</ul>';
					}
					echo '</div>';
				}
			} // query view
		
			// —————————————————————————————————————— IP DIAGNOSIS VIEW ——————————————————————————————————————
		
			else if ($viewType == 'ipdiag')
			{
				echo '<a href="?view=index" title="' . unbtrc('overview link') . '" class="roundnavlink">' . GetIcon('roundback') . '</a>';
		
				echo '<h1>IP address diagnosis for ' . $ip . ' <span class="domain">tòmò@' . $_SERVER['SERVER_NAME'] . '</span></h1>';

				echo '</div>'; // #page-header

				echo '<div id="page-content">';

				$host = false;
				$ip = $_GET['ip'];
				if (preg_match('_^[0-9a-f.:]+$_i', $ip))
				{
					$host = gethostbyaddr($ip);
				}
				
				if ($host !== false)
				{
					echo '<h2>Hostname</h2>';
					echo '<div>' . gethostbyaddr($ip) . '</div>';
		
					echo '<h2>Traceroute</h2>';
					echo '<pre style="line-height: 125%; border: solid 1px #cccccc; padding-left: 2px; max-height: 15em; overflow: auto;">';
					// TODO: apply htmlspecialchars
					system('traceroute -q 1 ' . $ip);
					echo '</pre>';
		
					echo '<h2>Whois</h2>';
					echo '<pre style="line-height: 125%; border: solid 1px #cccccc; padding-left: 2px; max-height: 25em; overflow: auto;">';
					// TODO: apply htmlspecialchars
					system('whois ' . $ip);
					echo '</pre>';
				}
				else
				{
					echo '<p class="error">Sorry, this is not a valid IP address. Maybe it has been hashed.</p>';
				}
			} // IP diagnosis view
		
			// —————————————————————————————————————— INFO AND MANAGEMENT VIEW ——————————————————————————————————————
		
			else if ($viewType == 'info')
			{
				echo '<a href="?view=index" title="' . unbtrc('overview link') . '" class="roundnavlink">' . GetIcon('roundback') . '</a>';
		
				echo '<h1>' . unbtrc('info and management') . ' <span class="domain">tòmò@' . $_SERVER['SERVER_NAME'] . '</span></h1>';

				echo '</div>'; // #page-header

				echo '<div id="page-content">';

				// —————————— Form handlers for creating custom variables ——————————
		
				if (isset($_POST['create-visitor-var']))
				{
					$name = $_POST['create-visitor-var'];
					try
					{
						$tomo->AddVisitorVar($name);
						echo '<p class="success">' . unbtr('custom visitor variable created', 'name', $name) . '</p>';
					}
					catch (UnbDatabaseException $ex)
					{
						echo '<p class="error">' . unbtr('custom visitor variable create error', 'name', $name) . '<br>' .
							htmlspecialchars($ex->getMessage()) . '</p>';
					}
				}
				if (isset($_POST['create-session-var']))
				{
					$name = $_POST['create-session-var'];
					try
					{
						$tomo->AddSessionVar($name);
						echo '<p class="success">' . unbtr('custom session variable created', 'name', $name) . '</p>';
					}
					catch (UnbDatabaseException $ex)
					{
						echo '<p class="error">' . unbtr('custom session variable create error', 'name', $name) . '<br>' .
							htmlspecialchars($ex->getMessage()) . '</p>';
					}
				}
				if (isset($_POST['create-pageview-var']))
				{
					$name = $_POST['create-pageview-var'];
					try
					{
						$tomo->AddPageviewVar($name);
						echo '<p class="success">' . unbtr('custom pageview variable created', 'name', $name) . '</p>';
					}
					catch (UnbDatabaseException $ex)
					{
						echo '<p class="error">' . unbtr('custom pageview variable create error', 'name', $name) . '<br>' .
							htmlspecialchars($ex->getMessage()) . '</p>';
					}
				}

				// —————————— tòmò database UI ——————————
		
				echo '<h2>' . unbtr('tomo database') . '</h2>';

				echo '<table>';
				echo '<tr><td>' . unbtrc('file size') . '</td><td class="right">';
				$fileName = dirname(__FILE__) . '/' . UnbConfiguration::Get('database.filename');
				echo unbds(filesize($fileName));
				echo '</td></tr>';
				echo '<tr><td>' . unbtrc('visitors') . '</td><td class="right">';
				echo unbnum(UnbDatabase::GetInstance()->QueryValue('select count(*) from "#visitor"'));
				echo '</td></tr>';
				echo '<tr><td>' . unbtrc('sessions') . '</td><td class="right">';
				echo unbnum(UnbDatabase::GetInstance()->QueryValue('select count(*) from "#session"'));
				echo '</td></tr>';
				echo '<tr><td>' . unbtrc('pageviews') . '</td><td class="right">';
				echo unbnum(UnbDatabase::GetInstance()->QueryValue('select count(*) from "#pageview"'));
				echo '</td></tr>';
				echo '<tr><td>' . unbtrc('texts') . '</td><td class="right">';
				echo unbnum(UnbDatabase::GetInstance()->QueryValue('select count(*) from "#text"'));
				echo '</td></tr>';
				echo '<tr><td>' . unbtrc('first entry') . '</td><td class="right">';
				$minPageviewTime = UnbDatabase::GetInstance()->QueryValue('select min("PageviewTime") from "#pageview"');
				if ($minPageviewTime)
				{
					echo unbtm(UnbDatabase::ConvertDateTime($minPageviewTime), UnbTimeFormat::Date);
				}
				else
				{
					echo unbtr('time:never');
				}
				echo '</td></tr>';
				echo '</table>';

				// —————————— Data files UI ——————————
		
				echo '<h2>' . unbtrc('data files status') . '</h2>';

				$geoIpPath = UnbConfiguration::Get('geoip.path', '');
				if (strlen($geoIpPath))
				{
					$geoIpDir = $geoIpPath;
					if (is_file($geoIpDir))
					{
						$geoIpDir = dirname($geoIpDir);
					}
				}
				else
				{
					$geoIpDir = dirname(__FILE__);
				}

				echo '<table>';
				echo '<tr><td>GeoIP</td><td>' . unbtrc('geoip.country-level database') . '</td><td>';
				
				$fileName = $geoIpDir . '/geolitecountry.bin';
				if (file_exists($fileName))
				{
					echo unbtr('updated on date', 'date', unbtm(filemtime($fileName), UnbTimeFormat::Date));
				}
				else
				{
					echo unbtr('not present');
				}
				echo '</td></tr>';
				echo '<tr><td class="same-group"></td><td>' . unbtrc('geoip.city-level database') . '</td><td>';
				$fileName = $geoIpDir . '/geolitecity.bin';
				if (file_exists($fileName))
				{
					echo unbtr('updated on date', 'date', unbtm(filemtime($fileName), UnbTimeFormat::Date));
				}
				else
				{
					echo unbtr('not present');
				}
				echo '</td></tr>';
				echo '<tr class="new-group"><td colspan="2">' . unbtrc('public suffix list') . '</td><td>';
				$fileName = PublicSuffix::GetSuffixDataFileName();
				if (file_exists($fileName))
				{
					echo unbtr('updated on date', 'date', unbtm(filemtime($fileName), UnbTimeFormat::Date));
				}
				else
				{
					echo unbtr('not present');
				}
				echo '</td></tr>';
				echo '</table>';
		
				// —————————— Custom variables UI ——————————
		
				$dbVisitorVars = Tomo::GetDbVisitorVars();
				$confVisitorVars = Tomo::GetConfVisitorVars();
		
				$dbSessionVars = Tomo::GetDbSessionVars();
				$confSessionVars = Tomo::GetConfSessionVars();
		
				$dbPageviewVars = Tomo::GetDbPageviewVars();
				$confPageviewVars = Tomo::GetConfPageviewVars();
		
				echo '<h2>' . unbtrc('custom variables') . '</h2>';

				if (count($confVisitorVars) ||
					count($dbVisitorVars) ||
					count($confSessionVars) ||
					count($dbSessionVars) ||
					count($confPageviewVars) ||
					count($dbPageviewVars))
				{
					echo '<table class="with-header">';

					echo '<tr>';
					echo '<th>' . unbtrc('customvar.scope') . '</th>';
					echo '<th>' . unbtrc('customvar.name') . '</th>';
					echo '<th>' . unbtrc('customvar.type') . '</th>';
					echo '<th>' . unbtrc('customvar.notes') . '</th>';
					echo '</tr>';

					// Visitor variables
					$count = 0;
					foreach ($confVisitorVars as $name => $type)
					{
						$count++;
						if ($count == 1)
							echo '<tr class="new-group"><td>' . unbtrc('visitor') . '</td>';
						else
							echo '<tr><td class="same-group"></td>';
						echo '</td><td>' . $name . '</td><td>' . $type . '</td><td>';
						if (!preg_match(Tomo::varNamePattern, $name))
						{
							echo '<strong>' . unbtrc('customvar.invalid name') . '</strong>';
						}
						else if (!array_key_exists($name, $dbVisitorVars))
						{
							echo '<form action="?view=info" id="form-create-v-' . $name . '" method="post" style="display: inline;">';
							echo '<input type="hidden" name="create-visitor-var" value="' . $name . '">';
							echo '<a href="javascript:undefined" onclick="document.getElementById(\'form-create-v-' . $name . '\').submit()">' . unbtrc('customvar.create in database') . '</a>';
							echo '</form>';
						}
						else
						{
							echo unbtr('customvar.ok');
						}
						echo '</td></tr>';
					}
					foreach ($dbVisitorVars as $name => $type)
					{
						if (array_key_exists($name, $confVisitorVars)) continue;   // Already displayed as configured

						$count++;
						if ($count == 1)
							echo '<tr class="new-group"><td>' . unbtrc('visitor') . '</td>';
						else
							echo '<tr><td class="same-group"></td>';
						echo '</td><td>' . $name . '</td><td>' . $type . '</td><td>';
						$msg = '';
						if (!array_key_exists($name, $confVisitorVars))
						{
							$msg .= '<em>' . unbtrc('customvar.in database not configured') . '</em>';
						}
						if (!preg_match(Tomo::varNamePattern, $name))
						{
							$msg .= ($msg ? ' – ' : '') . '<strong>' . unbtrc('customvar.invalid name') . '</strong>';
						}
						echo $msg;
						echo '</td></tr>';
					}

					// Session variables
					$count = 0;
					foreach ($confSessionVars as $name => $type)
					{
						$count++;
						if ($count == 1)
							echo '<tr class="new-group"><td>' . unbtrc('session') . '</td>';
						else
							echo '<tr><td class="same-group"></td>';
						echo '</td><td>' . $name . '</td><td>' . $type . '</td><td>';
						if (!preg_match(Tomo::varNamePattern, $name))
						{
							echo '<strong>' . unbtrc('customvar.invalid name') . '</strong>';
						}
						else if (!array_key_exists($name, $dbSessionVars))
						{
							echo '<form action="?view=info" id="form-create-s-' . $name . '" method="post" style="display: inline;">';
							echo '<input type="hidden" name="create-session-var" value="' . $name . '">';
							echo '<a href="javascript:undefined" onclick="document.getElementById(\'form-create-s-' . $name . '\').submit()">' . unbtrc('customvar.create in database') . '</a>';
							echo '</form>';
						}
						else
						{
							echo unbtr('customvar.ok');
						}
						echo '</td></tr>';
					}
					foreach ($dbSessionVars as $name => $type)
					{
						if (array_key_exists($name, $confSessionVars)) continue;   // Already displayed as configured

						$count++;
						if ($count == 1)
							echo '<tr class="new-group"><td>' . unbtrc('session') . '</td>';
						else
							echo '<tr><td class="same-group"></td>';
						echo '</td><td>' . $name . '</td><td>' . $type . '</td><td>';
						$msg = '';
						if (!array_key_exists($name, $confSessionVars))
						{
							$msg .= '<em>' . unbtrc('customvar.in database not configured') . '</em>';
						}
						if (!preg_match(Tomo::varNamePattern, $name))
						{
							$msg .= ($msg ? ' – ' : '') . '<strong>' . unbtrc('customvar.invalid name') . '</strong>';
						}
						echo $msg;
						echo '</td></tr>';
					}

					// Pageview variables
					$count = 0;
					foreach ($confPageviewVars as $name => $type)
					{
						$count++;
						if ($count == 1)
							echo '<tr class="new-group"><td>' . unbtrc('pageview') . '</td>';
						else
							echo '<tr><td class="same-group"></td>';
						echo '</td><td>' . $name . '</td><td>' . $type . '</td><td>';
						if (!preg_match(Tomo::varNamePattern, $name))
						{
							echo '<strong>' . unbtrc('customvar.invalid name') . '</strong>';
						}
						else if (!array_key_exists($name, $dbPageviewVars))
						{
							echo '<form action="?view=info" id="form-create-p-' . $name . '" method="post" style="display: inline;">';
							echo '<input type="hidden" name="create-pageview-var" value="' . $name . '">';
							echo '<a href="javascript:undefined" onclick="document.getElementById(\'form-create-p-' . $name . '\').submit()">' . unbtrc('customvar.create in database') . '</a>';
							echo '</form>';
						}
						else
						{
							echo unbtr('customvar.ok');
						}
						echo '</td></tr>';
					}
					foreach ($dbPageviewVars as $name => $type)
					{
						if (array_key_exists($name, $confPageviewVars)) continue;   // Already displayed as configured

						$count++;
						if ($count == 1)
							echo '<tr class="new-group"><td>' . unbtrc('pageview') . '</td>';
						else
							echo '<tr><td class="same-group"></td>';
						echo '</td><td>' . $name . '</td><td>' . $type . '</td><td>';
						$msg = '';
						if (!array_key_exists($name, $confPageviewVars))
						{
							$msg .= '<em>' . unbtrc('customvar.in database not configured') . '</em>';
						}
						if (!preg_match(Tomo::varNamePattern, $name))
						{
							$msg .= ($msg ? ' – ' : '') . '<strong>' . unbtrc('customvar.invalid name') . '</strong>';
						}
						echo $msg;
						echo '</td></tr>';
					}

					echo '</table>';
				}
				else
				{
					echo '<p>' . unbtrc('customvar.none') . '</p>';
				}
			} // info and management view
		
			// ———————————————————————————————————————— DEFAULT VIEW ————————————————————————————————————————
		
			else
			{
				echo '<h1>' . unbtr('tomo viewer title') . ' <span class="domain">' . $_SERVER['SERVER_NAME'] . '</span></h1>';
				
				echo '</div>'; // #page-header

				echo '<div id="page-content">';

				// Security checks
				if (!strlen(UnbConfiguration::Get('access.password')))
				{
					echo '<p class="warning">' . unbtr('warning.access password unset') . '</p>';
				}
				if (UnbConfiguration::Get('access.password') == '1234')
				{
					echo '<p class="warning">' . unbtr('warning.access password default') . '</p>';
				}
				if (!strlen(UnbConfiguration::Get('optout.password')))
				{
					echo '<p class="warning">' . unbtr('warning.opt-out password unset') . '</p>';
				}
				if (UnbConfiguration::Get('optout.password') == '1234')
				{
					echo '<p class="warning">' . unbtr('warning.opt-out password default') . '</p>';
				}
		
				echo '<p>' . unbtr('tomo viewer intro') . '</p>';
		
				echo '<div class="index-links">';
				echo '<a href="?view=visitorlog" class="novis main-nav-link mnl-visitorlog">' . unbtrc('visitor log') . '</a>';
				echo '<a href="?view=query" class="novis main-nav-link mnl-query">' . unbtrc('query view') . '</a>';
				echo '<a href="?view=info" class="novis main-nav-link mnl-info">' . unbtrc('info and management') . '</a>';
				echo '</div>'; // .index-links
				
				echo '<div style="clear: both;"></div>';
				
				echo '<h2>' . unbtrc('admin opt-out cookie') . '</h2>';
				
				$optoutCookieName = UnbConfiguration::Get('cookie.name.optout', 'tomo_optout');
				$optoutToken = $_COOKIE[$optoutCookieName];
				$password = UnbConfiguration::Get('optout.password');
				$cookiePath = UnbConfiguration::Get('cookie.path');
				$saltLength = 8;
		
				if ($_POST['delete_optout'])
				{
					$optoutToken = null;
					setcookie(
						$optoutCookieName,
						'*',
						1,
						$cookiePath);
				}
		
				if (isset($optoutToken))
				{
					// Validate given optout token
					$salt = substr($optoutToken, 0, $saltLength);
					if ($optoutToken != $salt . md5($salt . $password))
					{
						// Invalid token, don't use it
						$optoutToken = null;
					}
				}
		
				if (!isset($optoutToken) && isset($_POST['set_optout']))
				{
					// Set optout cookie.
					// The salt is the first 8 characters of some random md5 hash. This implementation is
					// simple and should be effective enough.
					$salt = substr(md5(mt_rand()), 0, $saltLength);
					$optoutToken = $salt . md5($salt . $password);
		
					setcookie(
						$optoutCookieName,
						$optoutToken,
						time() + intval(intval(UnbConfiguration::Get('cookie.months', 12)) * 30.44 * 86400),
						$cookiePath);
				}
		
				if (!isset($optoutToken))
				{
					echo '<form method="post">';
					echo '<p class="optout-cookie-unset">';
					echo unbtr('admin opt-out cookie.unset') . '<br>';
					echo '<input type="submit" name="set_optout" value="' . unbtrc('admin opt-out cookie.do set') . '">';
					echo '</p>';
					echo '</form>';
				}
				else
				{
					echo '<form method="post">';
					echo '<p class="optout-cookie-set">';
					echo unbtr('admin opt-out cookie.set') . '<br>';
					echo '<input type="submit" name="delete_optout" value="' . unbtrc('admin opt-out cookie.do unset') . '">';
					echo '</p>';
					echo '</form>';
				}

				echo '<h2>' . unbtr('app info') . '</h2>';
				
				echo '<p>';
				$tomoVersion = '???';
				if (file_exists('version.inc.php'))
				{
					include 'version.inc.php';
				}
				echo '<a href="http://tomo.unclassified.de/" class="novis">tòmò web analytics</a><br>';
				echo unbtrc('version') . ' ' . $tomoVersion . '<br>';
				echo '© Yves Goergen<br>';
				echo unbtrc('current language') . unbcol() . ' ' . UnbLocale::GetNativeName() . ' ' . unbpar(UnbLocale::GetCode());
				echo '</p>';
			} // default view
		
		} // if isset($accessToken)

		$dbStat = UnbDatabase::GetInstance()->GetStatistics();
		
		echo '<p class="grey">';
		if (isset($accessToken))
		{
			echo 'Execution time: ' . round((microtime(true) - self::$t0) * 1000, 2) . ' ms – Database time: ' .
				round($dbStat['totalTime'] * 1000, 2) . ' ms (' . $dbStat['queryCount'] . ' queries)';
		}
		echo '</p>';

		echo '</div>'; // #page-content

		echo '</body>';
		echo '</html>';
	}
	
} // class TomoViewer

?>
